@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA and W3C, 1996-2005
+ *  (c) COPYRIGHT INRIA and W3C, 1996-2007
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -674,9 +674,13 @@ ThotBool IsXMLName (const char *path)
   if (!strcasecmp (suffix, "xml") ||
       !strcasecmp (suffix, "xht") ||
       !strcasecmp (suffix, "xtd") ||
+      !strcasecmp (suffix, "xtl") ||
       !strcmp (suffix, "xhtm") ||
       !strcmp (suffix, "xhtml") ||
       !strcmp (suffix, "smi") ||
+      !strcmp (suffix, "xsd") |
+      !strcmp (suffix, "xslt") |
+      !strcmp (suffix, "xsl") |
       !strcmp (suffix, "zsl"))
     ret = TRUE;
   else if (!strcmp (suffix, "gz"))
@@ -686,9 +690,52 @@ ThotBool IsXMLName (const char *path)
       if (!strcasecmp (suffix, "xml") ||
           !strcasecmp (suffix, "xht") ||
           !strcasecmp (suffix, "xtd") ||
+          !strcasecmp (suffix, "xtl") ||
           !strcmp (suffix, "xhtm") ||
           !strcmp (suffix, "xhtml") ||
-          !strcmp (suffix, "smi") |
+          !strcmp (suffix, "xsd") |
+          !strcmp (suffix, "xslt") |
+          !strcmp (suffix, "xsl") |
+          !strcmp (suffix, "smi"))
+        ret = TRUE;
+      else
+        ret = FALSE;
+    }
+  else
+    ret = FALSE;
+
+  TtaFreeMemory (temppath);
+  TtaFreeMemory (suffix);
+  return (ret);
+}
+
+/*----------------------------------------------------------------------
+           IsXMLStruct                                                
+  returns TRUE if path points to an XML transformation or schema.
+  ----------------------------------------------------------------------*/
+ThotBool IsXMLStruct (const char *path)
+{
+  char        *temppath;
+  char        *suffix;
+  ThotBool    ret;
+
+  if (!path)
+    return (FALSE);
+
+  temppath = TtaStrdup ((char *)path);
+  suffix = (char *)TtaGetMemory (strlen (path) + 1);
+  TtaExtractSuffix (temppath, suffix);
+
+  if (!strcmp (suffix, "xsd") |
+      !strcmp (suffix, "xslt") |
+      !strcmp (suffix, "xsl"))
+    ret = TRUE;
+  else if (!strcmp (suffix, "gz"))
+    {
+      /* take into account compressed files */
+      TtaExtractSuffix (temppath, suffix);       
+      if (!strcmp (suffix, "xsd") |
+          !strcmp (suffix, "xslt") |
           !strcmp (suffix, "xsl"))
         ret = TRUE;
       else
@@ -719,13 +766,13 @@ ThotBool IsXTiger (const char *path)
   suffix = (char *)TtaGetMemory (strlen (path) + 1);
   TtaExtractSuffix (temppath, suffix);
 
-  if (!strcasecmp (suffix, "xtd"))
+  if (!strcasecmp (suffix, "xtd") || !strcasecmp (suffix, "xtl"))
     ret = TRUE;
   else if (!strcmp (suffix, "gz"))
     {
       /* take into account compressed files */
       TtaExtractSuffix (temppath, suffix);       
-      if (!strcasecmp (suffix, "xtd"))
+      if (!strcasecmp (suffix, "xtd") || !strcasecmp (suffix, "xtl"))
         ret = TRUE;
       else
         ret = FALSE;
@@ -2274,7 +2321,8 @@ void         SimplifyUrl (char **url)
 
 
 /*----------------------------------------------------------------------
-  NormalizeFile normalizes local names.                             
+  NormalizeFile normalizes local names.
+  convertion is AM_CONV_NONE or AM_CONV_ALL
   Return TRUE if target and src differ.                           
   ----------------------------------------------------------------------*/
 ThotBool NormalizeFile (char *src, char *target, ConvertionType convertion)
