@@ -83,6 +83,29 @@ void SetAttributeStringValueWithUndo (Element el, int att, char* value)
 #endif /* TEMPLATES */
 }
 
+/*----------------------------------------------------------------------
+Returns the value of a string attribute without copy it 
+----------------------------------------------------------------------*/
+void GiveAttributeStringValueFromNum (Element el, int att, char* buff, int* sz)
+{
+#ifdef TEMPLATES
+  AttributeType attType;
+  Attribute     attribute;
+  int           size;
+
+  attType.AttrSSchema = TtaGetElementType(el).ElSSchema;
+  attType.AttrTypeNum = att;
+  attribute = TtaGetAttribute(el, attType);
+  
+  size = TtaGetTextAttributeLength(attribute);
+  TtaGiveTextAttributeValue (attribute, buff, &size);
+  buff[size] = EOS;
+  if(sz)
+    *sz = size;
+#endif /* TEMPLATES */
+}
+
+
 
 /*----------------------------------------------------------------------
 Returns the value of a string attribute 
@@ -91,14 +114,18 @@ char *GetAttributeStringValueFromNum (Element el, int att, int* sz)
 {
 #ifdef TEMPLATES
 	AttributeType attType;
+  Attribute     attribute;
+  char         *aux;
+  int           size;
+
 	attType.AttrSSchema = TtaGetElementType(el).ElSSchema;
 	attType.AttrTypeNum = att;
+	attribute = TtaGetAttribute(el, attType);
 	
-	Attribute attribute = TtaGetAttribute(el, attType);
-	
-	int size = TtaGetTextAttributeLength(attribute);
-	char *aux = (char*) TtaGetMemory(size+1);
+	size = TtaGetTextAttributeLength(attribute);
+	aux = (char*) TtaGetMemory(size+1);
 	TtaGiveTextAttributeValue (attribute, aux, &size);
+  aux[size] = EOS;
   if(sz)
     *sz = size;
 	return aux;
@@ -115,7 +142,9 @@ char *GetAttributeStringValue (Element el, Attribute attribute, int* sz)
 #ifdef TEMPLATES
 	int size = TtaGetTextAttributeLength(attribute);
 	char *aux = (char*) TtaGetMemory(size+1);
+
 	TtaGiveTextAttributeValue (attribute, aux, &size);
+  aux[size] = EOS;
   if(sz)
     *sz = size;
 	return aux;
