@@ -37,19 +37,14 @@ BEGIN_EVENT_TABLE(NewTemplateDocDlgWX, AmayaDialog)
   EVT_BUTTON( XRCID("wxID_OK"), NewTemplateDocDlgWX::OnCreateButton )
   EVT_BUTTON( XRCID("wxID_CLEAR"), NewTemplateDocDlgWX::OnClearButton )
   EVT_BUTTON( XRCID("wxID_CANCEL"), NewTemplateDocDlgWX::OnCancelButton )
-  EVT_BUTTON( XRCID("wxID_BUTTON_INSTANCENAME"), NewTemplateDocDlgWX::OnInstanceFilenameButton )
-  EVT_BUTTON( XRCID("wxID_BUTTON_TEMPLATEDIR"), NewTemplateDocDlgWX::OnDirNameButton )
+  //  EVT_BUTTON( XRCID("wxID_BUTTON_INSTANCENAME"), NewTemplateDocDlgWX::OnInstanceFilenameButton )
+  EVT_BUTTON( XRCID("wxID_BUTTON_TEMPLATE"), NewTemplateDocDlgWX::OnInstanceFilenameButton )
   EVT_BUTTON( XRCID("wxID_BUTTON_INSTANCEDIR"), NewTemplateDocDlgWX::OnDirNameButton )
   EVT_TEXT_ENTER( XRCID("wxID_FILENAME"), NewTemplateDocDlgWX::OnCreateButton )
 
-  EVT_TEXT_ENTER( XRCID("wxID_TEMPLATEDIRNAME"), NewTemplateDocDlgWX::OnCreateButton )
   EVT_TEXT_ENTER( XRCID("wxID_TEMPLATEFILENAME"), NewTemplateDocDlgWX::OnCreateButton )
-  EVT_TEXT_ENTER( XRCID("wxID_INSTANCEDIR"), NewTemplateDocDlgWX::OnCreateButton )
   EVT_TEXT_ENTER( XRCID("wxID_INSTANCEFILENAME"), NewTemplateDocDlgWX::OnCreateButton )
   EVT_COMBOBOX( XRCID("wxID_TEMPLATEFILENAME"), NewTemplateDocDlgWX::OnTemplatenameSelected )
-  EVT_TEXT( XRCID("wxID_TEMPLATEDIRNAME"), NewTemplateDocDlgWX::OnText_TemplateDirName )
-  EVT_TEXT( XRCID("wxID_INSTANCE"), NewTemplateDocDlgWX::OnText_InstanceDirName )
-  EVT_TEXT( XRCID("wxID_INSTANCEFILENAME"), NewTemplateDocDlgWX::OnText_InstanceFilename )
   EVT_CLOSE( NewTemplateDocDlgWX::OnClose )
   END_EVENT_TABLE()
 
@@ -67,7 +62,6 @@ NewTemplateDocDlgWX::NewTemplateDocDlgWX ( int ref,
                                            wxWindow* parent,
                                            int doc,
                                            const wxString & title,
-                                           const wxString & templateDir,
                                            const wxString & filter,
                                            int * p_last_used_filter
                                            ) :
@@ -75,6 +69,7 @@ NewTemplateDocDlgWX::NewTemplateDocDlgWX ( int ref,
   ,m_LockUpdateFlag(false)
   ,m_pLastUsedFilter(p_last_used_filter)
 {
+  char      buffer[MAX_LENGTH];
   wxXmlResource::Get()->LoadDialog(this, parent, wxT("NewTemplateDocDlgWX"));
   Waiting = 1;
   MyRef = ref;
@@ -83,13 +78,9 @@ NewTemplateDocDlgWX::NewTemplateDocDlgWX ( int ref,
   SetTitle( title );
   GetTemplatesConf ();
 
-  //XRCCTRL(*this, "wxID_LABEL_TEMPLATE", wxStaticText)->SetLabel( TtaConvMessageToWX( TtaGetMessage(AMAYA,AM_TEMPLATE) ));
-  XRCCTRL(*this, "wxID_LABEL_TEMPLATEDIRNAME", wxStaticText)->SetLabel( TtaConvMessageToWX( TtaGetMessage(AMAYA, AM_DIRECTORY) ));
-  XRCCTRL(*this, "wxID_LABEL_INSTANCEDIR", wxStaticText)->SetLabel( TtaConvMessageToWX( TtaGetMessage(AMAYA, AM_DIRECTORY) ));
-  
   XRCCTRL(*this, "wxID_LABEL_INSTANCEFILENAME", wxStaticText)->SetLabel( TtaConvMessageToWX( TtaGetMessage(AMAYA,AM_TEMPLATE_INSTANCE) ));
-  XRCCTRL(*this, "wxID_BUTTON_TEMPLATEDIR", wxBitmapButton)->SetToolTip(TtaConvMessageToWX(TtaGetMessage(LIB,TMSG_SEL)));
-  XRCCTRL(*this, "wxID_BUTTON_INSTANCENAME", wxBitmapButton)->SetToolTip(TtaConvMessageToWX(TtaGetMessage(AMAYA,AM_BROWSE)));
+  XRCCTRL(*this, "wxID_BUTTON_TEMPLATE", wxBitmapButton)->SetToolTip(TtaConvMessageToWX(TtaGetMessage(LIB,TMSG_SEL)));
+  //XRCCTRL(*this, "wxID_BUTTON_INSTANCENAME", wxBitmapButton)->SetToolTip(TtaConvMessageToWX(TtaGetMessage(AMAYA,AM_BROWSE)));
   XRCCTRL(*this, "wxID_BUTTON_INSTANCEDIR", wxBitmapButton)->SetToolTip(TtaConvMessageToWX(TtaGetMessage(LIB,TMSG_SEL)));
 
   XRCCTRL(*this, "wxID_LABEL_TITLE", wxStaticText)->SetLabel(TtaConvMessageToWX(TtaGetMessage(AMAYA, AM_BM_TITLE)));
@@ -113,21 +104,35 @@ NewTemplateDocDlgWX::NewTemplateDocDlgWX ( int ref,
   wxChar dir_sep = DIR_SEP;
   m_DirSep = wxString(dir_sep);
 
-  // set the default templates directory
-  XRCCTRL(*this, "wxID_TEMPLATEDIRNAME", wxTextCtrl)->SetValue(templateDir);
-
-
-  // Update the template combobox with templates in directory "templateDir"
-#if defined(_WINDOWS) || defined(_MACOS)
-  UpdateTemplateFromDir ();
-#endif /* _WINDOWS  || MACOS */
-
   // set the default instance path
-  wxString homedir = TtaGetHomeDir();
-  wxString filename = TtaConvMessageToWX("New.html");
+  wxString homedir = TtaGetHomeDir ();
+  // generate the default name
+  strcpy (buffer, TtaGetMessage (AMAYA, AM_NEW));
+  strcat (buffer, ".html");
+  wxString filename = TtaConvMessageToWX (buffer);
   wxString   full_path = homedir + m_DirSep + filename;
+
+  // propose a new filename
+  strncpy( buffer, (const char*)full_path.mb_str(wxConvUTF8), MAX_LENGTH - 1);
+  buffer[MAX_LENGTH - 1] = EOS;
+  if (TtaFileExist(buffer))
+    {
+      int        i = 1, len;
+      len = strlen (buffer);
+      while (buffer[len] != '.')
+        len--;
+      do
+        {
+          sprintf (&buffer[len], "%d.html", i);
+          i++;
+        }
+      while (TtaFileExist(buffer));
+      full_path = TtaConvMessageToWX (buffer);
+    }
   XRCCTRL(*this, "wxID_INSTANCEFILENAME", wxTextCtrl)->SetValue(full_path);
-  UpdateDirFromString (full_path);
+
+  // Update the template combobox with pre-declared templates
+  UpdateTemplateList ();
   SetAutoLayout( TRUE );
 }
 
@@ -144,28 +149,6 @@ NewTemplateDocDlgWX::~NewTemplateDocDlgWX()
 }
 
 
-/*----------------------------------------------------------------------
-  UpdateDirFromString
-  params:
-  returns:
-  ----------------------------------------------------------------------*/
-void NewTemplateDocDlgWX::UpdateDirFromString( const wxString & full_path)
-{
-  if (!m_LockUpdateFlag)
-    {
-      m_LockUpdateFlag = true;
-      if (!full_path.StartsWith(_T("http")))
-        {
-          int end_slash_pos = full_path.Find(DIR_SEP, true);
-          wxString dir_value = full_path.SubString(0, end_slash_pos);
-          XRCCTRL(*this, "wxID_INSTANCEDIR", wxTextCtrl)->SetValue(dir_value);
-        }
-      else
-        XRCCTRL(*this, "wxID_INSTANCEDIR", wxTextCtrl)->Clear();
-      m_LockUpdateFlag = false;
-    }
-}
-
 /*----------------------------------------------------------------------
   OnInstanceFilenameButton called when the user want to change the filename
   to an existing one with the filebrowser
@@ -175,24 +158,49 @@ void NewTemplateDocDlgWX::UpdateDirFromString( const wxString & full_path)
 void NewTemplateDocDlgWX::OnInstanceFilenameButton( wxCommandEvent& event )
 {
   wxFileDialog  *p_dlg;
+  wxString       url, file_value;
+  int id = event.GetId();
+  int temp_id = wxXmlResource::GetXRCID(_T("wxID_BUTTON_TEMPLATE"));
 
   // Create a generic filedialog
-  p_dlg = new wxFileDialog(this,
-                           TtaConvMessageToWX( TtaGetMessage (AMAYA, AM_OPEN_URL) ),
-                           _T(""), _T(""), m_Filter,
-                           wxOPEN | wxCHANGE_DIR);
-  wxString url = XRCCTRL(*this, "wxID_INSTANCEFILENAME", wxTextCtrl)->GetValue( );
+  if (id == temp_id)
+    {
+      p_dlg = new wxFileDialog(this,
+                               TtaConvMessageToWX( TtaGetMessage (AMAYA, AM_OPEN_URL) ),
+                               _T(""), _T(""), _T("Templates (*.xtd)|*.xtd"),
+                               wxOPEN | wxCHANGE_DIR);
+      url = XRCCTRL(*this, "wxID_TEMPLATEFILENAME", wxComboBox)->GetValue( );
+    }
+  else
+    {
+      p_dlg = new wxFileDialog(this,
+                               TtaConvMessageToWX( TtaGetMessage (AMAYA, AM_OPEN_URL) ),
+                               _T(""), _T(""), m_Filter,
+                               wxOPEN | wxCHANGE_DIR);
+      url = XRCCTRL(*this, "wxID_INSTANCEFILENAME", wxTextCtrl)->GetValue( );
+      p_dlg->SetFilterIndex(*m_pLastUsedFilter);
+    }
+
+  // set an initial path
   if (url.StartsWith(_T("http")) ||
       url.StartsWith(TtaConvMessageToWX((TtaGetEnvString ("THOTDIR")))))
     p_dlg->SetDirectory(wxGetHomeDir());
   else
-    p_dlg->SetPath(url);
-  p_dlg->SetFilterIndex(*m_pLastUsedFilter);
-
+    {
+      file_value = url.AfterLast (DIR_SEP);
+      p_dlg->SetPath (url);
+      p_dlg->SetFilename (file_value);
+    }
+      
   if (p_dlg->ShowModal() == wxID_OK)
     {
-      *m_pLastUsedFilter = p_dlg->GetFilterIndex();
-      UpdateInstanceFromString( p_dlg->GetPath() );
+      if (id == temp_id)
+        XRCCTRL(*this, "wxID_TEMPLATEFILENAME", wxComboBox)->SetValue( p_dlg->GetPath() );
+      else
+        {
+          *m_pLastUsedFilter = p_dlg->GetFilterIndex();
+          UpdateInstanceFromString( p_dlg->GetPath() );
+        }
     }
   p_dlg->Destroy();
 }
@@ -202,7 +210,7 @@ void NewTemplateDocDlgWX::OnInstanceFilenameButton( wxCommandEvent& event )
   params:
   returns:
   ----------------------------------------------------------------------*/
-void NewTemplateDocDlgWX::UpdateTemplateFromDir ()
+void NewTemplateDocDlgWX::UpdateTemplateList ()
 {
   Prop_Templates       prop = GetProp_Templates();
   Prop_Templates_Path *path = prop.FirstPath;
@@ -217,63 +225,19 @@ void NewTemplateDocDlgWX::UpdateTemplateFromDir ()
       XRCCTRL(*this, "wxID_TEMPLATEFILENAME", wxComboBox)->SetValue( TtaConvMessageToWX(""));
       while (path)
         {
-		  value = TtaConvMessageToWX(path->Path);
+          value = TtaConvMessageToWX(path->Path);
           XRCCTRL(*this, "wxID_TEMPLATEFILENAME", wxComboBox)->Append(value);
-		  if (!initialized)
-		  {
-            XRCCTRL(*this, "wxID_TEMPLATEFILENAME", wxComboBox)->SetStringSelection(value);
-            initialized = true;
-		  }
+          if (!initialized)
+            {
+              XRCCTRL(*this, "wxID_TEMPLATEFILENAME", wxComboBox)->SetStringSelection(value);
+              initialized = true;
+            }
           path = path->NextPath;
         }
-
-      value = XRCCTRL(*this, "wxID_TEMPLATEDIRNAME", wxTextCtrl)->GetValue();
-      if (!value.StartsWith(_T("http")))
-        {
-          // get the directory list when it is a local directory
-#ifdef _WINDOWS
-          if (value.IsEmpty())
-            value = _T("C:\\");
-#endif /* _WINDOWS */
-          if (value.Last() != m_DirSep)
-            value += m_DirSep;
-          wxDir::GetAllFiles(value, &templateList, _T("*.xtd"), wxDIR_FILES);
-          XRCCTRL(*this, "wxID_TEMPLATEFILENAME", wxComboBox)->Append( templateList );
-          if (!initialized && !templateList.IsEmpty())
-            XRCCTRL(*this, "wxID_TEMPLATEFILENAME", wxComboBox)->SetStringSelection(templateList.Item(0));
-        }
-      // adding paths form preferences
       m_LockUpdateFlag = false;
    }
 }
 
-/*----------------------------------------------------------------------
-  UpdateInstanceFromDir
-  params:
-  returns:
-  ----------------------------------------------------------------------*/
-void NewTemplateDocDlgWX::UpdateInstanceFromDir ()
-{
-  wxString value, dir_value, filename_value;
-  
-  if (!m_LockUpdateFlag)
-    {
-      m_LockUpdateFlag = true;
-      value = XRCCTRL (*this, "wxID_INSTANCEFILENAME", wxTextCtrl)->GetValue();
-      int end_slash_pos = value.Find(DIR_SEP, true);
-      filename_value = value.SubString(end_slash_pos+1, value.Length());
-      dir_value = XRCCTRL(*this, "wxID_INSTANCEDIR", wxTextCtrl)->GetValue();
-#ifdef _WINDOWS
-      if (dir_value.IsEmpty())
-        dir_value = _T("C:\\");
-#endif /* _WINDOWS */
-      if (dir_value.Last() != m_DirSep)
-        dir_value += m_DirSep;
-      
-      m_LockUpdateFlag = false;
-      XRCCTRL(*this, "wxID_INSTANCEFILENAME", wxTextCtrl)->SetValue(dir_value + filename_value);
-    }
-}
 
 /*----------------------------------------------------------------------
   UpdateInstanceFromString
@@ -294,7 +258,6 @@ void NewTemplateDocDlgWX::UpdateInstanceFromString( const wxString & full_path )
         }
       else
         XRCCTRL(*this, "wxID_INSTANCEFILENAME", wxTextCtrl)->Clear();
-      UpdateDirFromString (full_path);
       m_LockUpdateFlag = false;
     }
 }
@@ -306,9 +269,10 @@ void NewTemplateDocDlgWX::UpdateInstanceFromString( const wxString & full_path )
   ----------------------------------------------------------------------*/
 void NewTemplateDocDlgWX::OnCreateButton( wxCommandEvent& event )
 {
-  wxString        value, title;
+  wxString        value, title, name;
   char            temp[MAX_LENGTH], docname[MAX_LENGTH];
   char            buffer[MAX_LENGTH];
+  int             end_pos;
 
   if (!Waiting)
     // avoid double click
@@ -328,20 +292,30 @@ void NewTemplateDocDlgWX::OnCreateButton( wxCommandEvent& event )
       title = XRCCTRL(*this, "wxID_TITLE", wxTextCtrl)->GetValue( );
       if (title.Len() == 0)
         {
+          // get the donument name as default document title
+          end_pos = value.Find(DIR_SEP, true);
+          if (end_pos)
+            title = value.Mid(end_pos+1, value.Length());
+          else
+            title = value;
+          end_pos = title.Find('.', true);
+          if (end_pos)
+            name = title.Mid(0, end_pos);
+          else
+            name = title;
           if (Waiting == 1)
             {
               // request the title
+              XRCCTRL (*this, "wxID_TITLE", wxTextCtrl)->SetValue(name);
+              // select the string
+              XRCCTRL(*this, "wxID_TITLE", wxTextCtrl)->SetSelection(0, -1);
               XRCCTRL(*this, "wxID_ERROR", wxStaticText)->SetLabel( TtaConvMessageToWX(TtaGetMessage (AMAYA, AM_MISSING_TITLE)));
               Waiting = 2;
               return;
             }
           else
             {
-              int end_slash_pos = value.Find(DIR_SEP, true);
-              m_LockUpdateFlag = true;
-              title = value.SubString(end_slash_pos+1, value.Length());
-              m_LockUpdateFlag = false;
-              strncpy( buffer, (const char*)title.mb_str(wxConvUTF8), MAX_LENGTH - 1);
+              strncpy( buffer, (const char*)name.mb_str(wxConvUTF8), MAX_LENGTH - 1);
               buffer[MAX_LENGTH - 1] = EOS;
               ThotCallback (BaseDialog + TitleText,  STRING_DATA, (char *)buffer);
             }
@@ -383,10 +357,7 @@ void NewTemplateDocDlgWX::OnCreateButton( wxCommandEvent& event )
   ----------------------------------------------------------------------*/
 void NewTemplateDocDlgWX::OnClearButton( wxCommandEvent& event )
 {
-  //XRCCTRL(*this, "wxID_TEMPLATEFILENAME", wxComboBox)->SetValue(_T(""));
-  //XRCCTRL(*this, "wxID_TEMPLATEDIRNAME", wxTextCtrl)->Clear();
   XRCCTRL(*this, "wxID_INSTANCEFILENAME", wxTextCtrl)->Clear();
-  XRCCTRL(*this, "wxID_INSTANCEDIR", wxTextCtrl)->Clear();
 }
 
 /*----------------------------------------------------------------------
@@ -418,41 +389,6 @@ void NewTemplateDocDlgWX::OnTemplatenameSelected( wxCommandEvent& event )
 }
 
 
-/*----------------------------------------------------------------------
-  Class:  NewTemplateDocDlgWX
-  Method:  OnText_InstanceFilename
-  Description:  update the filename
-  ----------------------------------------------------------------------*/
-void NewTemplateDocDlgWX::OnText_InstanceFilename( wxCommandEvent& event )
-{
-  wxString   full_path = XRCCTRL(*this, "wxID_INSTANCEFILENAME", wxTextCtrl)->GetValue();
-  UpdateDirFromString (full_path);
-  event.Skip();
-}
-
-
-/*----------------------------------------------------------------------
-  Class:  NewTemplateDocDlgWX
-  Method:  OnText_Templatename_Browse
-  Description: Select a template from its path
-  ----------------------------------------------------------------------*/
-void NewTemplateDocDlgWX::OnText_TemplateDirName (wxCommandEvent& event )
-{
-  UpdateTemplateFromDir ();
-  event.Skip();
-}
-
-/*----------------------------------------------------------------------
-  Class:  NewTemplateDocDlgWX
-  Method:  OnText_InstanceDirName
-  Description: Select a template from its path
-  ----------------------------------------------------------------------*/
-void NewTemplateDocDlgWX::OnText_InstanceDirName (wxCommandEvent& event )
-{
-  UpdateInstanceFromDir ();
-  event.Skip();
-}
-
 /*----------------------------------------------------------------------
   Class:  NewTemplateDocDlgWX
   Method:  OnDirNameButton
@@ -460,31 +396,41 @@ void NewTemplateDocDlgWX::OnText_InstanceDirName (wxCommandEvent& event )
   ----------------------------------------------------------------------*/
 void NewTemplateDocDlgWX::OnDirNameButton (wxCommandEvent& event)
 {
-  int id = event.GetId();
-  int temp_id = wxXmlResource::GetXRCID(_T("wxID_BUTTON_TEMPLATEDIR"));
-  wxDirDialog *p_dlg = new wxDirDialog(this);
-  //p_dlg->SetStyle(p_dlg->GetStyle() | wxDD_NEW_DIR_BUTTON);
-  if (id == temp_id)
+  wxString        url, dir_value;
+  wxDirDialog    *p_dlg;
+  char            buffer[MAX_LENGTH];
+  int             len, end_pos;
+
+  if (!m_LockUpdateFlag)
     {
-      p_dlg->SetPath(XRCCTRL(*this, "wxID_TEMPLATEDIRNAME", wxTextCtrl)->GetValue());
+      m_LockUpdateFlag = true;
+      url = XRCCTRL(*this, "wxID_INSTANCEFILENAME", wxTextCtrl)->GetValue( );
+      end_pos = url.Find(DIR_SEP, true);
+      if (end_pos)
+        dir_value = url.Mid(0, end_pos+1);
+      else
+        dir_value = url;
+      p_dlg = new wxDirDialog(this);
+      // set an initial path
+      if (url.StartsWith(_T("http")) ||
+          url.StartsWith(TtaConvMessageToWX((TtaGetEnvString ("THOTDIR")))))
+          p_dlg->SetPath(wxGetHomeDir());
+      else
+          p_dlg->SetPath(dir_value);
+
       if (p_dlg->ShowModal() == wxID_OK)
         {
-          XRCCTRL(*this, "wxID_TEMPLATEDIRNAME", wxTextCtrl)->SetValue( p_dlg->GetPath() );
-          p_dlg->Destroy();
+          dir_value =  p_dlg->GetPath();
+          strcpy (buffer, (const char*)dir_value.mb_str(wxConvUTF8));
+          len = strlen (buffer);
+          if (buffer[len-1] == DIR_SEP)
+            buffer[len-1] = EOS;
+          dir_value = url.Mid(end_pos);
+          strcat (buffer, (const char*)dir_value.mb_str(wxConvUTF8));          
+          XRCCTRL(*this, "wxID_INSTANCEFILENAME", wxTextCtrl)->SetValue(TtaConvMessageToWX(buffer));
         }
-      else
-        p_dlg->Destroy();
-    }
-  else
-    {
-      p_dlg->SetPath(XRCCTRL(*this, "wxID_INSTANCEDIR", wxTextCtrl)->GetValue());
-      if (p_dlg->ShowModal() == wxID_OK)
-        {
-          XRCCTRL(*this, "wxID_INSTANCEDIR", wxTextCtrl)->SetValue( p_dlg->GetPath() );
-          p_dlg->Destroy();
-        }
-      else
-        p_dlg->Destroy();
+      p_dlg->Destroy();
+      m_LockUpdateFlag = false;
     }
 }
 
