@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996.
+ *  (c) COPYRIGHT INRIA, 1996-2005
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -9,10 +9,9 @@
    printstr.c : Impression en clair sur la sortie standard du contenu 
    d'un fichier .STR contenant un schema de structure.  
 
-   V. Quint     Mars 1985
+   V. Quint
   ----------------------------------------------------------------------*/
 
-
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
@@ -21,7 +20,7 @@
 #include "strmsg.h"
 #include "message.h"
 
-#define THOT_EXPORT
+#define THOT_EXPORT extern
 #include "platform_tv.h"
 
 static PtrSSchema   pSchStr;
@@ -30,43 +29,25 @@ static int          a;
 static int          i, min, nb;
 static Name         fn;
 static int          r;
-static int          PremRegle;
-static boolean      Prem;
-static boolean      optionh = False;
-static boolean      optionl = False;
-static int          NbCommentaire;
+static int          FirstRule;
+static ThotBool     First;
+static ThotBool     optionh = False;
+static ThotBool     optionl = False;
+static int          CommentNum;
 static int          STR;	/* Identification des messages Str */
 PtrSSchema          pSc1;
-SRule              *pRe1;
-TtAttribute        *pAt1;
+PtrTtAttribute      pAt1;
 
 #include "readstr_f.h"
 
-#ifdef __STDC__
-extern char        *TtaGetEnvString (char *);
-extern void         TtaInitializeAppRegistry (char *);
+extern char*      TtaGetEnvString (char*);
+extern void         TtaInitializeAppRegistry (char*);
 extern void         TtaSaveAppRegistry (void);
 
-#else
-extern char        *TtaGetEnvString ();
-extern void         TtaInitializeAppRegistry ();
-extern void         TtaSaveAppRegistry ();
-
-#endif /* __STDC__ */
-
 /*----------------------------------------------------------------------
    wrcar ecrit un caractere.                                       
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrcar (unsigned char ch)
-
-#else  /* __STDC__ */
-static void         wrcar (ch)
-unsigned char       ch;
-
-#endif /* __STDC__ */
-
 {
    int                 c;
 
@@ -155,20 +136,10 @@ unsigned char       ch;
      }
 }
 
-
 /*----------------------------------------------------------------------
    writenb ecrit un nombre.                                        
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         writenb (int n)
-
-#else  /* __STDC__ */
-static void         writenb (n)
-int                 n;
-
-#endif /* __STDC__ */
-
 {
    if (n >= 100000)
       printf ("%6d", n);
@@ -184,20 +155,10 @@ int                 n;
       printf ("%1d", n);
 }
 
-
 /*----------------------------------------------------------------------
    wrnom ecrit une chaine de caracteres.                           
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrnom (Name n)
-
-#else  /* __STDC__ */
-static void         wrnom (n)
-Name                n;
-
-#endif /* __STDC__ */
-
 {
    int                 i;
 
@@ -209,107 +170,63 @@ Name                n;
      }
 }
 
-
 /*----------------------------------------------------------------------
    wrnomregle ecrit un nom de regle.                               
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrnomregle (int r)
-
-#else  /* __STDC__ */
-static void         wrnomregle (r)
-int                 r;
-
-#endif /* __STDC__ */
-
 {
-   if (pSchStr->SsRule[r - 1].SrName[0] == '\0')
+   if (pSchStr->SsRule->SrElem[r - 1]->SrName[0] == '\0')
       /* Name vide, on ecrit son numero */
      {
 	printf ("Ident");
 	writenb (r);
      }
    else
-      wrnom (pSchStr->SsRule[r - 1].SrName);
+      wrnom (pSchStr->SsRule->SrElem[r - 1]->SrName);
 }
 
-
 /*----------------------------------------------------------------------
-   DebutComment debute un commentaire.                             
+   OpenComment debute un commentaire.                             
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
-static void         DebutComment ()
-
-#else  /* __STDC__ */
-static void         DebutComment ()
-#endif				/* __STDC__ */
-
+static void         OpenComment ()
 {
-   if (NbCommentaire == 0)
+   if (CommentNum == 0)
       printf ("{ ");
-   NbCommentaire++;
+   CommentNum++;
 }
 
-
 /*----------------------------------------------------------------------
-   FinComment finit un commentaire.                                
+   CloseComment finit un commentaire.                                
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
-static void         FinComment ()
-
-#else  /* __STDC__ */
-static void         FinComment ()
-#endif				/* __STDC__ */
-
+static void         CloseComment ()
 {
-   NbCommentaire--;
-   if (NbCommentaire == 0)
+   CommentNum--;
+   if (CommentNum == 0)
       printf (" }");
 }
 
 /*----------------------------------------------------------------------
    wrNULL ecrit NULL.                                              
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrNULL ()
-
-#else  /* __STDC__ */
-static void         wrNULL ()
-#endif				/* __STDC__ */
-
 {
 
-   DebutComment ();
+   OpenComment ();
    printf ("NULL");
-   FinComment ();
+   CloseComment ();
 }
 
-
 /*----------------------------------------------------------------------
    wrnomreglext ecrit le nom de la regle de numero r. Cette regle  
    fait partie du schema de structure traite' si N est nul.
    Sinon elle fait partie du schema de nom N.              
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrnomreglext (int r, Name N)
-
-#else  /* __STDC__ */
-static void         wrnomreglext (r, N)
-int                 r;
-Name                N;
-
-#endif /* __STDC__ */
-
 {
    if (N[0] == '\0')
      {
-	if (pSchStr->SsRule[r - 1].SrConstruct == CsPairedElement)
-	   if (!pSchStr->SsRule[r - 1].SrFirstOfPair)
+	if (pSchStr->SsRule->SrElem[r - 1]->SrConstruct == CsPairedElement)
+	   if (!pSchStr->SsRule->SrElem[r - 1]->SrFirstOfPair)
 	      printf ("Second ");
 	wrnomregle (r);
      }
@@ -319,10 +236,10 @@ Name                N;
 	   printf ("******");
 	else
 	  {
-	     if (pSchStrExt->SsRule[r - 1].SrConstruct == CsPairedElement)
-		if (!pSchStrExt->SsRule[r - 1].SrFirstOfPair)
+	     if (pSchStrExt->SsRule->SrElem[r - 1]->SrConstruct == CsPairedElement)
+		if (!pSchStrExt->SsRule->SrElem[r - 1]->SrFirstOfPair)
 		   printf ("Second ");
-	     wrnom (pSchStrExt->SsRule[r - 1].SrName);
+	     wrnom (pSchStrExt->SsRule->SrElem[r - 1]->SrName);
 	  }
 	printf ("(");
 	wrnom (N);
@@ -330,26 +247,19 @@ Name                N;
      }
 }
 
-
 /*----------------------------------------------------------------------
    wrNumExcept ecrit le numero d'exception Num.                    
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrNumExcept (int Num)
-
-#else  /* __STDC__ */
-static void         wrNumExcept (Num)
-int                 Num;
-
-#endif /* __STDC__ */
-
 {
    switch (Num)
 	 {
 	    case ExcNoCut:
 	       printf ("NoCut");
 	       break;
+	    case ExcCanCut:
+	       printf ("CanCut");
+	       break;
 	    case ExcNoCreate:
 	       printf ("NoCreate");
 	       break;
@@ -380,6 +290,21 @@ int                 Num;
 	    case ExcInvisible:
 	       printf ("Invisible");
 	       break;
+	    case ExcEventAttr:
+	       printf ("EventAttr");
+	       break;
+	    case ExcCssId:
+	       printf ("CssId");
+	       break;
+	    case ExcCssClass:
+	       printf ("CssClass");
+	       break;
+	    case ExcCssPseudoClass:
+	       printf ("CssPseudoClass");
+	       break;
+	    case ExcCssBackground:
+	       printf ("CssBackground");
+	       break;
 	    case ExcNoMove:
 	       printf ("NoMove");
 	       break;
@@ -389,6 +314,9 @@ int                 Num;
 	    case ExcNoSelect:
 	       printf ("NoSelect");
 	       break;
+	    case ExcSelectParent:
+	       printf ("SelectParent");
+	       break;
 	    case ExcMoveResize:
 	       printf ("MoveResize");
 	       break;
@@ -396,7 +324,16 @@ int                 Num;
 	       printf ("NoSpellCheck");
 	       break;
 	    case ExcIsDraw:
-	       printf ("NoIsDraw");
+	       printf ("IsDraw");
+	       break;
+	    case ExcIsImg:
+	       printf ("IsImg");
+	       break;
+	    case ExcIsMap:
+	       printf ("IsMap");
+	       break;
+	    case ExcNoShowBox:
+	       printf ("NoShowBox");
 	       break;
 	    case ExcGraphCreation:
 	       printf ("GraphCreation");
@@ -422,47 +359,90 @@ int                 Num;
 	    case ExcActiveRef:
 	       printf ("ActiveRef");
 	       break;
+	    case ExcNoPaginate:
+	       printf ("NoPaginate");
+	       break;
+	    case ExcImportLine:
+	       printf ("ImportLine");
+	       break;
+	    case ExcImportParagraph:
+	       printf ("ImportParagraph");
+	       break;
+	    case ExcParagraphBreak:
+	       printf ("ParagraphBreak");
+	       break;
+	    case ExcIsBreak:
+	       printf ("IsBreak");
+	       break;
+	    case ExcHighlightChildren:
+	       printf ("HighlightChildren");
+	       break;
+	    case ExcExtendedSelection:
+	       printf ("ExtendedSelection");
+	       break;
+	    case ExcClickableSurface:
+	       printf ("ClickableSurface");
+	       break;
+	    case ExcReturnCreateNL:
+	       printf ("ReturnCreateNL");
+	       break;
+	    case ExcReturnCreateWithin:
+	       printf ("ReturnCreateWithin");
+	       break;
+	    case ExcIsTable:
+	       printf ("IsTable");
+	       break;
+	    case ExcIsRow:
+	       printf ("IsRow");
+	       break;
+	    case ExcIsColHead:
+	       printf ("IsColHead");
+	       break;
+	    case ExcIsCell:
+	       printf ("IsCell");
+	       break;
+	    case ExcIsCaption:
+	       printf ("IsCaption");
+	       break;
+	    case ExcShadow:
+	       printf ("Shadow");
+	       break;
+	    case ExcNewPercentWidth:
+	       printf ("NewPercentWidth");
+	       break;
+	    case ExcColRef:
+	       printf ("ColRef");
+	       break;
+	    case ExcColSpan:
+	       printf ("ColSpan");
+	       break;
+	    case ExcRowSpan:
+	       printf ("RowSpan");
+	       break;
+
 	    default:
 	       printf ("%2d", Num);
 	       break;
 	 }
 }
 
-
 /*----------------------------------------------------------------------
    Wdefine ecrit #define.                                          
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         Wdefine ()
-
-#else  /* __STDC__ */
-static void         Wdefine ()
-#endif				/* __STDC__ */
-
 {
    printf ("#define ");
 }
 
-
 /*----------------------------------------------------------------------
    wrattr ecrit un attribut.                                       
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrattr (int a)
-
-#else  /* __STDC__ */
-static void         wrattr (a)
-int                 a;
-
-#endif /* __STDC__ */
-
 {
    int                 j;
-   TtAttribute        *pAt1;
+   PtrTtAttribute      pAt1;
 
-   pAt1 = &pSchStr->SsAttribute[a - 1];
+   pAt1 = pSchStr->SsAttribute->TtAttr[a - 1];
    if (optionh)
      {
 	if (pAt1->AttrGlobal)
@@ -531,42 +511,24 @@ int                 a;
    pAt1->AttrGlobal = True;
 }
 
-
 /*----------------------------------------------------------------------
    wrext ecrit le mot extern si la regle de numero r est une      
    reference d'inclusion de document externe.              
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrext (int r)
-
-#else  /* __STDC__ */
-static void         wrext (r)
-int                 r;
-
-#endif /* __STDC__ */
-
 {
-   SRule              *pRe1;
+   PtrSRule         pRe1;
 
-   pRe1 = &pSchStr->SsRule[r - 1];
+   pRe1 = pSchStr->SsRule->SrElem[r - 1];
    if (pRe1->SrRefImportedDoc)
       if (pRe1->SrConstruct == CsReference)
 	 printf (" extern");
 }
 
-
 /*----------------------------------------------------------------------
    welembase ecrit un element de base.                             
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         welembase ()
-
-#else  /* __STDC__ */
-static void         welembase ()
-#endif				/* __STDC__ */
-
 {
    PtrSSchema          pSc1;
    int                 r;
@@ -602,7 +564,7 @@ static void         welembase ()
 
    Wdefine ();
    wrnom (pSc1->SsName);
-   r = Picture + 1;
+   r = tt_Picture + 1;
    printf ("_EL_");
    wrnomregle (r);
    printf (" ");
@@ -617,36 +579,33 @@ static void         welembase ()
    printf (" ");
    writenb (r);
    printf ("\n");
-}
 
+   Wdefine ();
+   wrnom (pSc1->SsName);
+   r = AnyType + 1;
+   printf ("_EL_");
+   wrnomregle (r);
+   printf (" ");
+   writenb (r);
+   printf ("\n");
+}
 
 /*----------------------------------------------------------------------
    wrrule ecrit une regle.                                         
    si pRegleExtens est non nul, il s'agit d'une regle d'extension  
   ----------------------------------------------------------------------*/
-
-
-#ifdef __STDC__
 static void         wrrule (int r, SRule * pRegleExtens)
-
-#else  /* __STDC__ */
-static void         wrrule (r, pRegleExtens)
-int                 r;
-SRule              *pRegleExtens;
-
-#endif /* __STDC__ */
-
 {
    int                 i;
    PtrSSchema          pSc1;
-   SRule              *pRe1;
-   TtAttribute        *pAt1;
+   PtrSRule            pRe1;
+   PtrTtAttribute      pAt1;
 
    pSc1 = pSchStr;
    if (pRegleExtens != NULL)
       pRe1 = pRegleExtens;
    else
-      pRe1 = &pSc1->SsRule[r - 1];
+      pRe1 = pSc1->SsRule->SrElem[r - 1];
    if (pRe1->SrConstruct != CsNatureSchema &&
        !(pRe1->SrConstruct == CsPairedElement && !pRe1->SrFirstOfPair))
      {
@@ -671,49 +630,50 @@ SRule              *pRegleExtens;
 	     writenb (r);
 	     printf ("\n");
 
-	     if (pRe1->SrNLocalAttrs > 0)
+	     if (pRe1->SrNLocalAttrs > 0 && pRe1->SrLocalAttr)
 	       {
-		  for (i = 1; i <= pRe1->SrNLocalAttrs; i++)
-		    {
-		       /*          if (pRe1->SrRequiredAttr[i - 1])
-		          printf("!"); */
-		       wrattr (pRe1->SrLocalAttr[i - 1]);
-		    }
+		 for (i = 0; i < pRe1->SrNLocalAttrs; i++)
+		   wrattr (pRe1->SrLocalAttr->Num[i]);
 	       }
 	     return;
 	  }
 	printf (" ");
-	DebutComment ();
+	OpenComment ();
 	writenb (r);
-	FinComment ();
+	CloseComment ();
 	putchar ('\t');
 	if (pRe1->SrRecursive)
 	  {
-	     DebutComment ();
+	     OpenComment ();
 	     printf ("Recursive");
-	     FinComment ();
+	     CloseComment ();
 	  }
 	if (pRegleExtens != NULL && pRe1->SrName[0] == '\0')
 	   printf ("Root");
 	else if (pRegleExtens != NULL)
 	   wrnom (pRe1->SrName);
 	else
-	   wrnomregle (r);
-	if (pRe1->SrNLocalAttrs > 0)
+	  {
+	    if (pRe1->SrConstruct == CsDocument)
+	      OpenComment ();
+	    wrnomregle (r);
+	  }
+	if (pRe1->SrNLocalAttrs > 0 &&
+	    pRe1->SrRequiredAttr && pRe1->SrLocalAttr)
 	  {
 	     printf (" (ATTR ");
-	     for (i = 1; i <= pRe1->SrNLocalAttrs; i++)
+	     for (i = 0; i < pRe1->SrNLocalAttrs; i++)
 	       {
-		  if (pRe1->SrRequiredAttr[i - 1])
+		  if (pRe1->SrRequiredAttr->Bln[i])
 		     printf ("!");
-		  wrattr (pRe1->SrLocalAttr[i - 1]);
+		  wrattr (pRe1->SrLocalAttr->Num[i]);
 		  if (i == pRe1->SrNLocalAttrs)
 		     printf (")");
 		  else
 		     printf (";\n%c%c", '\t', '\t');
 	       }
 	  }
-	if (pRegleExtens == NULL)
+	if (pRegleExtens == NULL && pRe1->SrConstruct != CsDocument)
 	   printf (" = ");
 	if (pRe1->SrNLocalAttrs > 0)
 	   printf ("\n%c%c", '\t', '\t');
@@ -820,11 +780,14 @@ SRule              *pRegleExtens;
 		 case CsPairedElement:
 		    printf ("PAIR");
 		    break;
+	         case CsAny:
+		    printf ("ANY");
+		    break;
 		 case CsExtensionRule:
-
+	         case CsDocument:
+	         case CsEmpty:
 		    break;
 	      }
-	/* Fin du switch SrConstruct */
 	/* ecrit les inclusions et les exclusions */
 	if (pRe1->SrNInclusions > 0)
 	  {
@@ -860,7 +823,7 @@ SRule              *pRegleExtens;
 	     printf ("%c%cWITH ", '\t', '\t');
 	     for (i = 1; i <= pRe1->SrNDefAttrs; i++)
 	       {
-		  pAt1 = &pSc1->SsAttribute[pRe1->SrDefAttr[i - 1] - 1];
+		  pAt1 = pSc1->SsAttribute->TtAttr[pRe1->SrDefAttr[i - 1] - 1];
 		  wrnom (pAt1->AttrName);
 		  if (pAt1->AttrType != AtReferenceAttr)
 		    {
@@ -888,23 +851,20 @@ SRule              *pRegleExtens;
 		     printf (", ");
 	       }
 	  }
-	printf (";\n");
+	if (pRe1->SrConstruct == CsDocument)
+	  {
+	    CloseComment ();
+	    printf ("\n");
+	  }
+	else
+	  printf (";\n");
      }
 }
 
-
 /*----------------------------------------------------------------------
    
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 void                printName (char *buffer)
-
-#else  /* __STDC__ */
-void                printName (buffer)
-char               *buffer;
-
-#endif /* __STDC__ */
-
 {
    int                 i = 0;
 
@@ -919,25 +879,16 @@ char               *buffer;
    printf (": \n");
 }
 
-
 /*----------------------------------------------------------------------
    main.                                                           
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 int                 main (int argc, char **argv)
-
-#else  /* __STDC__ */
-int                 main (argc, argv)
-int                 argc;
-char              **argv;
-
-#endif /* __STDC__ */
-
 {
+   PtrSRule            pRe1;
    char               *exec, c;
    int                 inputfile = False;
    int                 k, l;
-   boolean             premattrlocal;
+   ThotBool            premattrlocal;
 
    TtaInitializeAppRegistry (argv[0]);
    STR = TtaGetMessageTable ("strdialogue", STR_MSG_MAX);
@@ -980,7 +931,7 @@ char              **argv;
 	      printf ("{ This generic structure exports elements }\n");
 	if (optionh)
 	   printf ("/* Types and attributes for the document type %s */\n", pSc1->SsName);
-	NbCommentaire = 0;
+	CommentNum = 0;
 	if (!optionh)
 	  {
 	     printf ("STRUCTURE ");
@@ -994,7 +945,8 @@ char              **argv;
 	     printf (";\n");
 	  }
 	/* ecrit les attributs globaux */
-	if (pSc1->SsNAttributes > 0 && pSc1->SsAttribute[0].AttrGlobal)
+	if (pSc1->SsNAttributes > 0 &&
+	    pSc1->SsAttribute->TtAttr[0]->AttrGlobal)
 	  {
 	     if (!optionh)
 		printf ("ATTR\n");
@@ -1003,15 +955,15 @@ char              **argv;
 	  }
 
 	for (i = 1; i <= pSc1->SsNAttributes; i++)
-	   if (pSc1->SsAttribute[i - 1].AttrGlobal)
+	   if (pSc1->SsAttribute->TtAttr[i - 1]->AttrGlobal)
 	     {
 		if (!optionh)
 		   printf ("   ");
-		pSc1->SsAttribute[i - 1].AttrGlobal = False;
+		pSc1->SsAttribute->TtAttr[i - 1]->AttrGlobal = False;
 		/* indique a wrattr qu'il faut */
 		/* ecrire les valeurs de l'attribut */
 		wrattr (i);
-		pSc1->SsAttribute[i - 1].AttrGlobal = True;
+		pSc1->SsAttribute->TtAttr[i - 1]->AttrGlobal = True;
 		if (!optionh)
 		   printf (";\n");
 	     }
@@ -1020,7 +972,7 @@ char              **argv;
 	  {
 	     premattrlocal = True;
 	     for (i = 1; i <= pSc1->SsNAttributes; i++)
-		if (!pSc1->SsAttribute[i - 1].AttrGlobal)
+		if (!pSc1->SsAttribute->TtAttr[i - 1]->AttrGlobal)
 		  {
 		     if (premattrlocal)
 		       {
@@ -1040,17 +992,18 @@ char              **argv;
 
 	/* ecrit les constantes */
 	r = MAX_BASIC_TYPE + 1;
-	if (pSc1->SsRule[r - 1].SrConstruct == CsConstant)
+	if (pSc1->SsRule->SrElem[r - 1]->SrConstruct == CsConstant)
+	  {
 	   if (!optionh)
 	     {
 		printf ("CONST\n");
-		while (pSc1->SsRule[r - 1].SrConstruct == CsConstant)
+		while (pSc1->SsRule->SrElem[r - 1]->SrConstruct == CsConstant)
 		  {
-		     pRe1 = &pSc1->SsRule[r - 1];
+		     pRe1 = pSc1->SsRule->SrElem[r - 1];
 		     printf (" ");
-		     DebutComment ();
+		     OpenComment ();
 		     writenb (r);
-		     FinComment ();
+		     CloseComment ();
 		     putchar ('\t');
 		     wrnomregle (r);
 		     printf (" = \'");
@@ -1067,48 +1020,28 @@ char              **argv;
 	   else
 	     {
 		printf ("\n/* Constants */\n");
-		while (pSc1->SsRule[r - 1].SrConstruct == CsConstant)
+		while (pSc1->SsRule->SrElem[r - 1]->SrConstruct == CsConstant)
 		  {
 		     wrrule (r, NULL);
 		     r++;
 		  }
 	     }
-	PremRegle = r;
-	/* ecrit les parametres */
-	Prem = True;
-	for (r = PremRegle; r <= pSc1->SsNRules; r++)
-	   if (pSc1->SsRule[r - 1].SrParamElem)
-	     {
-		if (Prem)
-		  {
-		     if (!optionh)
-			printf ("PARAM\n");
-		     else
-			printf ("\n/* Parameters */\n");
-		     Prem = False;
-		  }
-		wrrule (r, NULL);
-	     }
+	  }
+	FirstRule = r;
 	/* ecrit les elements structures */
-	if (pSc1->SsNRules >= PremRegle)
+	if (pSc1->SsNRules >= FirstRule)
+	  {
 	   if (optionh)
 	      printf ("\n/* Elements */\n");
 	   else
 	      printf ("STRUCT\n");
-	for (r = PremRegle; r <= pSc1->SsNRules; r++)
+	  }
+	for (r = FirstRule; r <= pSc1->SsNRules; r++)
 	  {
-	     pRe1 = &pSc1->SsRule[r - 1];
-	     /* saute les parametres, les elements associes, les */
-	     /* elements Extern et Included et les unites */
-	     if (!pRe1->SrParamElem &&
-		 !pRe1->SrAssocElem &&
-		 !pRe1->SrRefImportedDoc &&
-		 !pRe1->SrUnitElem)
-		/* ignore les regles listes ajoutees pour les elements associes */
-		if (pRe1->SrConstruct != CsList)
-		   wrrule (r, NULL);
-		else if (!pSc1->SsRule[pRe1->SrListItem - 1].SrAssocElem)
-		   wrrule (r, NULL);
+	     pRe1 = pSc1->SsRule->SrElem[r - 1];
+	     /* saute les elements Extern et Included et les unites */
+	     if (!pRe1->SrRefImportedDoc && !pRe1->SrUnitElem)
+	       wrrule (r, NULL);
 	  }
 	/* ecrit les regles d'extension */
 	if (pSc1->SsExtension && pSc1->SsNExtensRules > 0)
@@ -1123,55 +1056,18 @@ char              **argv;
 		  wrrule (r, pRe1);
 	       }
 	  }
-	/* ecrit les elements associes */
-	Prem = True;
-	for (r = PremRegle; r <= pSc1->SsNRules; r++)
-	   if (pSc1->SsRule[r - 1].SrAssocElem)
-	     {
-		if (Prem)
-		  {
-		     if (!optionh)
-			printf ("ASSOC\n");
-		     else
-			printf ("\n/* Associated elements */\n");
-		     Prem = False;
-		  }
-		wrrule (r, NULL);
-	     }
-	if (!Prem)
-	   /* il y a au moins un element associe', on ecrit les regles listes */
-	   /* ajoutees pour les elements associes */
-	  {
-	     if (!optionh)
-	       {
-		  DebutComment ();
-		  printf ("lists of associated elements:\n");
-	       }
-	     for (r = PremRegle; r <= pSc1->SsNRules; r++)
-	       {
-		  pRe1 = &pSc1->SsRule[r - 1];
-		  if (pRe1->SrConstruct == CsList)
-		     if (pSc1->SsRule[pRe1->SrListItem - 1].SrAssocElem)
-			wrrule (r, NULL);
-	       }
-	     if (!optionh)
-	       {
-		  FinComment ();
-		  printf ("\n");
-	       }
-	  }
 	/* ecrit les unites exportees */
-	Prem = True;
-	for (r = PremRegle; r <= pSc1->SsNRules; r++)
-	   if (pSc1->SsRule[r - 1].SrUnitElem)
+	First = True;
+	for (r = FirstRule; r <= pSc1->SsNRules; r++)
+	   if (pSc1->SsRule->SrElem[r - 1]->SrUnitElem)
 	     {
-		if (Prem)
+		if (First)
 		  {
 		     if (!optionh)
 			printf ("UNITS\n");
 		     else
 			printf ("\n/* Units */\n");
-		     Prem = False;
+		     First = False;
 		  }
 		wrrule (r, NULL);
 	     }
@@ -1179,16 +1075,16 @@ char              **argv;
 	if (!optionh)
 	   if (pSc1->SsExport)
 	     {
-		Prem = True;
+		First = True;
 		printf ("THOT_EXPORT\n");
-		for (r = PremRegle; r <= pSc1->SsNRules; r++)
+		for (r = FirstRule; r <= pSc1->SsNRules; r++)
 		  {
-		     pRe1 = &pSc1->SsRule[r - 1];
+		     pRe1 = pSc1->SsRule->SrElem[r - 1];
 		     if (pRe1->SrExportedElem)
 		       {
-			  if (!Prem)
+			  if (!First)
 			     printf (",\n");
-			  Prem = False;
+			  First = False;
 			  printf ("   ");
 			  wrnomregle (r);
 			  if (pRe1->SrExportContent == 0)
@@ -1210,7 +1106,7 @@ char              **argv;
 		/* ecrit d'abord les exceptions de tous les attributs */
 		for (a = 1; a <= pSc1->SsNAttributes; a++)
 		  {
-		     pAt1 = &pSc1->SsAttribute[a - 1];
+		     pAt1 = pSc1->SsAttribute->TtAttr[a - 1];
 		     if (pAt1->AttrFirstExcept > 0)
 			/* cet attribut a des exceptions */
 		       {
@@ -1230,16 +1126,18 @@ char              **argv;
 		/* ecrit les exceptions des types d'elements */
 		for (r = 1; r <= pSc1->SsNRules; r++)
 		  {
-		     pRe1 = &pSc1->SsRule[r - 1];
+		     pRe1 = pSc1->SsRule->SrElem[r - 1];
 		     if (pRe1->SrFirstExcept > 0)
 			/* ce type a des exceptions */
 		       {
 			  printf ("   ");
 			  if (pRe1->SrConstruct == CsPairedElement)
+			    {
 			     if (pRe1->SrFirstOfPair)
 				printf ("First ");
 			     else
 				printf ("Second ");
+			    }
 			  wrnom (pRe1->SrName);
 			  printf (": ");
 			  for (i = pRe1->SrFirstExcept; i <= pRe1->SrLastExcept; i++)
@@ -1273,14 +1171,14 @@ char              **argv;
 	/* ecrit en commentaire les natures importees */
 	if (!optionh)
 	  {
-	     Prem = True;
-	     for (r = PremRegle; r <= pSc1->SsNRules; r++)
-		if (pSc1->SsRule[r - 1].SrConstruct == CsNatureSchema)
+	     First = True;
+	     for (r = FirstRule; r <= pSc1->SsNRules; r++)
+		if (pSc1->SsRule->SrElem[r - 1]->SrConstruct == CsNatureSchema)
 		  {
-		     if (Prem)
+		     if (First)
 		       {
 			  printf ("{ External structures used: ");
-			  Prem = False;
+			  First = False;
 		       }
 		     printf ("\n");
 		     printf ("   ");
@@ -1288,19 +1186,19 @@ char              **argv;
 		     putchar ('\t');
 		     wrnomregle (r);
 		  }
-	     if (!Prem)
+	     if (!First)
 		printf (" }\n");
 	  }
 	else
 	  {
-	     Prem = True;
-	     for (r = PremRegle; r <= pSc1->SsNRules; r++)
-		if (pSc1->SsRule[r - 1].SrConstruct == CsNatureSchema)
+	     First = True;
+	     for (r = FirstRule; r <= pSc1->SsNRules; r++)
+		if (pSc1->SsRule->SrElem[r - 1]->SrConstruct == CsNatureSchema)
 		  {
-		     if (Prem)
+		     if (First)
 		       {
 			  printf ("\n/* Imported natures */\n");
-			  Prem = False;
+			  First = False;
 		       }
 		     Wdefine ();
 		     wrnom (pSc1->SsName);
@@ -1341,19 +1239,19 @@ char              **argv;
      {
 	printf ("document\n\ntranslation\n\n");
 	for (k = 0; k < pSchStr->SsNRules; k++)
-	   printName (&(pSchStr->SsRule[k].SrName[0]));
+	   printName (&(pSchStr->SsRule->SrElem[k]->SrName[0]));
 
 	for (k = 0; k < pSchStr->SsNExtensRules; k++)
 	   if (pSchStr->SsExtensBlock->EbExtensRule[k].SrName[0] != '\0')
 	      printName (&(pSchStr->SsExtensBlock->EbExtensRule[k].SrName[0]));
 	for (k = 0; k < pSchStr->SsNAttributes; k++)
-	   printName (&(pSchStr->SsAttribute[k].AttrName[0]));
+	   printName (&(pSchStr->SsAttribute->TtAttr[k]->AttrName[0]));
 
 	for (k = 0; k < pSchStr->SsNAttributes; k++)
 	  {
-	     if (pSchStr->SsAttribute[k].AttrType == AtEnumAttr)
-		for (l = 0; l < pSchStr->SsAttribute[k].AttrNEnumValues; l++)
-		   printName (&(pSchStr->SsAttribute[k].AttrEnumValue[l][0]));
+	     if (pSchStr->SsAttribute->TtAttr[k]->AttrType == AtEnumAttr)
+		for (l = 0; l < pSchStr->SsAttribute->TtAttr[k]->AttrNEnumValues; l++)
+		   printName (&(pSchStr->SsAttribute->TtAttr[k]->AttrEnumValue[l][0]));
 	  }
      }
    TtaSaveAppRegistry ();
