@@ -110,6 +110,11 @@ extern void TtaRemoveAccessKey (Document doc, unsigned int key);
   ----------------------------------------------------------------------*/
 extern void TtaUpdateAttrMenu (Document document);
 
+/*----------------------------------------------------------------------
+  TtaIsTextInserting returns the TextInserting status
+  ----------------------------------------------------------------------*/
+extern ThotBool TtaIsTextInserting ();
+
 /*----------------------------------------------------------------------
   TtaExecuteMenuAction execute the corresponding menu action.
   ----------------------------------------------------------------------*/
