@@ -129,107 +129,111 @@
 #define TMSG_ATTR_TO_SEARCH		124
 #define TMSG_TYPE_TO_SEARCH		125
 #define TMSG_NATURES		126
-#define TMSG_VALUE_OF_ATTR		127
-#define TMSG_UNKNOWN_ATTR		128
-#define TMSG_CANCEL		129
-#define TMSG_ERROR_PS_TRANSLATION		130
-#define TMSG_UNKNOWN_PRINTER		131
-#define TMSG_CANNOT_CREATE_PS		132
-#define TMSG_OVERLINE		133
-#define TMSG_DOCUMENT_FORMAT		134
-#define TMSG_EVENTS		135
-#define TMSG_DEL_ATTR		136
-#define TMSG_CPBUTTON_1		137
-#define TMSG_CPBUTTON_2		138
-#define TMSG_CPMOREBUTTON		139
-#define TMSG_CPGETBUTTON		140
-#define TMSG_CPCOLORBUTTON		141
-#define TMSG_CPCOLORBG		142
-#define TMSG_CPCOLORFG		143
-#define TMSG_CPCOLORSWITCH		144
-#define TMSG_NO_ELEMENT		145
-#define TMSG_What		146
-#define TMSG_Pass_Without		147
-#define TMSG_Replace_With		148
-#define TMSG_Replace_Without		149
-#define TMSG_Keep_Unknown		150
-#define TMSG_Pass_With		151
-#define TMSG_Specials		152
-#define TMSG_Correct		153
-#define TMSG_Romans		154
-#define TMSG_Ignore		155
-#define TMSG_Capitals		156
-#define TMSG_Arabics		157
-#define TMSG_NO_PARAM		158
-#define TMSG_NO_KEYBOARD		159
-#define TMSG_NO_SAVE		160
-#define TMSG_REFUSE_WORD		161
-#define TMSG_ADD_WORD		162
-#define TMSG_END_CHECK		163
-#define TMSG_STOP_CHECK		164
-#define TMSG_NO_SELECT		165
-#define TMSG_OK_SAVE		166
-#define TMSG_NOT_AVAILABLE		167
-#define TMSG_ATTACHDETACH		168
-#define TMSG_TOOLS		169
-#define TMSG_ATTR_MANDATORY		170
-#define TMSG_REFRESH		171
-#define TMSG_FORMATLEFT		172
-#define TMSG_FORMATRIGHT		173
-#define TMSG_FORMATCENTER		174
-#define TMSG_FORMATJUSTIFY		175
-#define TMSG_APPLY_CLASS		176
-#define TMSG_SEL_CLASS		177
-#define TMSG_SEL		178
-#define TMSG_DISCARD		179
-#define TMSG_MATHML		180
-#define TMSG_MISC		181
-#define TMSG_BINARY_REL		182
-#define TMSG_BINARY_REL_NEG		183
-#define TMSG_OPERATOR		184
-#define TMSG_GREEK_CAP		185
-#define TMSG_ARROW		186
-#define TMSG_SPECHAR		187
-#define TMSG_BUTTON_INTERRUPT		188
-#define TMSG_BUTTON_PREVIOUS		189
-#define TMSG_BUTTON_NEXT		190
-#define TMSG_BUTTON_RELOAD		191
-#define TMSG_BUTTON_SAVE		192
-#define TMSG_BUTTON_PRINT		193
-#define TMSG_BUTTON_SEARCH		194
-#define TMSG_BUTTON_HOME		195
-#define TMSG_BUTTON_ITALICS		196
-#define TMSG_BUTTON_BOLD		197
-#define TMSG_BUTTON_CODE		198
-#define TMSG_BUTTON_IMG		199
-#define TMSG_BUTTON_H1		200
-#define TMSG_BUTTON_H2		201
-#define TMSG_BUTTON_H3		202
-#define TMSG_BUTTON_UL		203
-#define TMSG_BUTTON_OL		204
-#define TMSG_BUTTON_DL		205
-#define TMSG_BUTTON_LINK		206
-#define TMSG_BUTTON_TABLE		207
-#define TMSG_LIB_WARNCTAB_INFO		208
-#define TMSG_LIB_WARNCTAB_CHECK		209
-#define TMSG_LIB_CLOSETABS		210
-#define TMSG_LIB_NOERROR_DOC		211
-#define TMSG_LIB_ERROR_DOC		212
-#define TMSG_LIB_BOXES		213
-#define TMSG_BUTTON_DIV		214
-#define TMSG_BUTTON_TARGET		215
-#define TMSG_INSERTION		216
-#define TMSG_DELETION		217
-#define TMSG_SUB		218
-#define TMSG_SUP		219
-#define TMSG_BUTTON_DT		220
-#define TMSG_BUTTON_DD		221
-#define TMSG_OBJECT		222
-#define TMSG_CSSStyle		223
-#define TMSG_EXPLORE		224
-#define TMSG_SplitHorizontally		225
-#define TMSG_SplitVertically		226
-#define TMSG_ShowPanel		227
-#define TMSG_LIB_MSG_MAX		228
+#define TMSG_ATTRIBUTE_NAME		127
+#define TMSG_ATTRIBUTE_VALUE		128
+#define TMSG_VALUE_OF_ATTR		129
+#define TMSG_UNKNOWN_ATTR		130
+#define TMSG_CANCEL		131
+#define TMSG_ERROR_PS_TRANSLATION		132
+#define TMSG_UNKNOWN_PRINTER		133
+#define TMSG_CANNOT_CREATE_PS		134
+#define TMSG_OVERLINE		135
+#define TMSG_DOCUMENT_FORMAT		136
+#define TMSG_EVENTS		137
+#define TMSG_DEL_ATTR		138
+#define TMSG_CPBUTTON_1		139
+#define TMSG_CPBUTTON_2		140
+#define TMSG_CPMOREBUTTON		141
+#define TMSG_CPGETBUTTON		142
+#define TMSG_CPCOLORBUTTON		143
+#define TMSG_CPCOLORBG		144
+#define TMSG_CPCOLORFG		145
+#define TMSG_CPCOLORSWITCH		146
+#define TMSG_NO_ELEMENT		147
+#define TMSG_What		148
+#define TMSG_Pass_Without		149
+#define TMSG_Replace_With		150
+#define TMSG_Replace_Without		151
+#define TMSG_Keep_Unknown		152
+#define TMSG_Pass_With		153
+#define TMSG_Specials		154
+#define TMSG_Correct		155
+#define TMSG_Romans		156
+#define TMSG_Ignore		157
+#define TMSG_Capitals		158
+#define TMSG_Arabics		159
+#define TMSG_NO_PARAM		160
+#define TMSG_NO_KEYBOARD		161
+#define TMSG_NO_SAVE		162
+#define TMSG_REFUSE_WORD		163
+#define TMSG_ADD_WORD		164
+#define TMSG_END_CHECK		165
+#define TMSG_STOP_CHECK		166
+#define TMSG_NO_SELECT		167
+#define TMSG_OK_SAVE		168
+#define TMSG_NOT_AVAILABLE		169
+#define TMSG_ATTACHDETACH		170
+#define TMSG_TOOLS		171
+#define TMSG_ATTR_MANDATORY		172
+#define TMSG_REFRESH		173
+#define TMSG_FORMATLEFT		174
+#define TMSG_FORMATRIGHT		175
+#define TMSG_FORMATCENTER		176
+#define TMSG_FORMATJUSTIFY		177
+#define TMSG_APPLY_CLASS		178
+#define TMSG_SEL_CLASS		179
+#define TMSG_SEL		180
+#define TMSG_DISCARD		181
+#define TMSG_MATHML		182
+#define TMSG_MISC		183
+#define TMSG_BINARY_REL		184
+#define TMSG_BINARY_REL_NEG		185
+#define TMSG_OPERATOR		186
+#define TMSG_GREEK_CAP		187
+#define TMSG_ARROW		188
+#define TMSG_SPECHAR		189
+#define TMSG_BUTTON_INTERRUPT		190
+#define TMSG_BUTTON_PREVIOUS		191
+#define TMSG_BUTTON_NEXT		192
+#define TMSG_BUTTON_RELOAD		193
+#define TMSG_BUTTON_SAVE		194
+#define TMSG_BUTTON_NEW		195
+#define TMSG_BUTTON_OPEN		196
+#define TMSG_BUTTON_PRINT		197
+#define TMSG_BUTTON_SEARCH		198
+#define TMSG_BUTTON_HOME		199
+#define TMSG_BUTTON_ITALICS		200
+#define TMSG_BUTTON_BOLD		201
+#define TMSG_BUTTON_CODE		202
+#define TMSG_BUTTON_IMG		203
+#define TMSG_BUTTON_H1		204
+#define TMSG_BUTTON_H2		205
+#define TMSG_BUTTON_H3		206
+#define TMSG_BUTTON_UL		207
+#define TMSG_BUTTON_OL		208
+#define TMSG_BUTTON_DL		209
+#define TMSG_BUTTON_LINK		210
+#define TMSG_BUTTON_TABLE		211
+#define TMSG_LIB_WARNCTAB_INFO		212
+#define TMSG_LIB_WARNCTAB_CHECK		213
+#define TMSG_LIB_CLOSETABS		214
+#define TMSG_LIB_NOERROR_DOC		215
+#define TMSG_LIB_ERROR_DOC		216
+#define TMSG_LIB_BOXES		217
+#define TMSG_BUTTON_DIV		218
+#define TMSG_BUTTON_TARGET		219
+#define TMSG_INSERTION		220
+#define TMSG_DELETION		221
+#define TMSG_SUB		222
+#define TMSG_SUP		223
+#define TMSG_BUTTON_DT		224
+#define TMSG_BUTTON_DD		225
+#define TMSG_OBJECT		226
+#define TMSG_CSSStyle		227
+#define TMSG_EXPLORE		228
+#define TMSG_SplitHorizontally		229
+#define TMSG_SplitVertically		230
+#define TMSG_ShowPanel		231
+#define TMSG_LIB_MSG_MAX		232
 
 #endif
