@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA 1996-2005
+ *  (c) COPYRIGHT INRIA 1996-2007
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -220,6 +220,20 @@ typedef enum
 
 typedef struct _PathSeg *PtrPathSeg;
 
+
+/* descriptor of the selection to do after redosplaying */
+typedef struct _SelectionDescriptor
+{
+  Element             SDElemSel;
+  Element             SDElemExt;
+  Attribute           SDAttribute;
+  int                 SDFirstChar;
+  int                 SDLastChar;
+  int                 SDCharExt;
+  ThotBool            SDSelActive;
+}
+SelectionDescriptor;
+
 /* Description of a SVG path segment */
 typedef struct _PathSeg
 {
