@@ -10,17 +10,17 @@
 
 THOT_EXPORT int	    numOfJobs;
 THOT_EXPORT PathBuffer   PSdir;
-THOT_EXPORT boolean      PaperPrint;
-THOT_EXPORT boolean      ManualFeed;
-THOT_EXPORT boolean      NewPaperPrint;
+THOT_EXPORT ThotBool     PaperPrint;
+THOT_EXPORT ThotBool     ManualFeed;
+THOT_EXPORT ThotBool     NewPaperPrint;
 THOT_EXPORT char         pPrinter[MAX_PATH];
 THOT_EXPORT Document     PrintingDoc;
 THOT_EXPORT char         PageSize[MAX_NAME_LENGTH];
-THOT_EXPORT int	    FirstPage;
-THOT_EXPORT int	    LastPage;
-THOT_EXPORT int	    NbCopies;
-THOT_EXPORT int	    Reduction;
-THOT_EXPORT int	    PagesPerSheet;
-THOT_EXPORT int	    Paginate;
+THOT_EXPORT int	         FirstPage;
+THOT_EXPORT int          LastPage;
+THOT_EXPORT int	         NbCopies;
+THOT_EXPORT int          Reduction;
+THOT_EXPORT int          PagesPerSheet;
+THOT_EXPORT int	         Paginate;
 
 #endif /* __PRINT_V_H__ */
