@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2005
+ *  (c) COPYRIGHT INRIA, 1996-2007
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -107,8 +107,6 @@
  * degrees to radians and so on...
  * All for EllipticSplit and/or GL_DrawArc
  */
-#define PRECISION float
-
 #ifndef M_PI
 #define M_PI 3.14159265358979323846
 #endif
@@ -118,8 +116,8 @@
 #endif
 
 /* ((A)*(M_PI/180.0)) */
-#define DEG_TO_RAD(A)   ((PRECISION)A)/57.29577957795135
-#define RAD_TO_DEG(A)   ((PRECISION)A)*57.29577957795135
+#define DEG_TO_RAD(A)   ((GLfloat)A)/57.29577957795135
+#define RAD_TO_DEG(A)   ((GLfloat)A)*57.29577957795135
 
 #define GL_EPSILON                 5.e-3
 #define IS_ZERO(arg)                    (fabs(arg)<1.e-20)
@@ -155,17 +153,16 @@
 /*for int  => (((int)N)<<1)*/
 #define P2(N) (N*N)
 /* Arc Precision Drawings */
-#define SLICES 360
-#define SLICES_SIZE 361
+#define SLICES 180
+#define SLICES_SIZE 181
 #define INTERVAL 0.02 /*1/FPS*/ /* should be 1/25 ... 1/50 */
 #define REALY(A) (A + FrameTable[frame].FrTopMargin)
 
 /*--------- STATICS ------*/
 /*Current Thickness*/
-static int      S_thick;
-static GLubyte  Opacity = 255;
 static GLubyte  FillOpacity = 255;
 static GLubyte  StrokeOpacity = 255;
+static GLfloat  S_thick = 0.;
 static int      X_Clip = 0;
 static int      Y_Clip = 0;
 static int      Width_Clip = 0;
@@ -261,7 +258,7 @@ void update_bg_colorGTK (int frame, int color)
   ----------------------------------------------------------------------*/
 void GL_SetOpacity (int opacity)
 {
-  Opacity = (GLubyte) ((opacity * 255)/1000);
+  FillOpacity = (GLubyte) ((opacity * 255)/1000);
 }
 
 /*----------------------------------------------------------------------
@@ -286,9 +283,11 @@ void GL_SetFillOpacity (int opacity)
 void GL_SetClipping (int x, int y, int width, int height)
 {
 #ifdef _WX
-  wxASSERT_MSG( x>=0 && y>=0 && width>=0 && height>=0, _T("GL_SetClipping : one clipping value is negative") );
+  wxASSERT_MSG( x>=0 && y>=0 && width>=0 && height>=0,
+                _T("GL_SetClipping : one clipping value is negative") );
 #endif /* _WX */
-  TTALOGDEBUG_4( TTA_LOG_DRAW, _T("GL_SetClipping : x=%d y=%d w=%d h=%d"), x, y, width, height );
+  TTALOGDEBUG_4( TTA_LOG_DRAW, _T("GL_SetClipping : x=%d y=%d w=%d h=%d"),
+                 x, y, width, height );
   glEnable (GL_SCISSOR_TEST);
   glScissor (x, y, width, height);
   if (Width_Clip == 0 && Height_Clip == 0)
@@ -334,11 +333,9 @@ void GL_SetForeground (int fg, ThotBool fillstyle)
   if (fillstyle)
     us_opac = FillOpacity;
   else
-    {
-      us_opac = StrokeOpacity;
-    }
-   TtaGiveThotRGB (fg, &red, &green, &blue);
-  glColor4ub ((GLubyte) red,  (GLubyte) green, (GLubyte) blue, (GLubyte) us_opac);    
+    us_opac = StrokeOpacity;
+  TtaGiveThotRGB (fg, &red, &green, &blue);
+  glColor4ub ((GLubyte) red,  (GLubyte) green, (GLubyte) blue, us_opac);
 }
 
 
@@ -347,60 +344,42 @@ void GL_SetForeground (int fg, ThotBool fillstyle)
   ----------------------------------------------------------------------*/
 void GL_SetPicForeground ()
 {
-  unsigned short us_opac;
+  GLubyte         us_opac;
 
   us_opac = (GLubyte) FillOpacity;
-  glColor4ub ((GLubyte) 255, (GLubyte) 255, (GLubyte) 255, (GLubyte) us_opac);    
+  glColor4ub ((GLubyte) 255, (GLubyte) 255, (GLubyte) 255, us_opac);
 }
 
+
 /*----------------------------------------------------------------------
   InitDrawing update the Graphic Context accordingly to parameters.
   The parameter fg indicates the drawing color
   ----------------------------------------------------------------------*/
 void InitDrawing (int style, int thick, int fg)
 {
-  if (style >= 5)
+  S_thick = (double)thick;
+  if (thick)
     {
-      /* solid */
-      if (thick)
-        {
-          S_thick = thick;
-          glLineWidth ((GLfloat) thick); 
-          glPointSize ((GLfloat) thick); 
-        }
-      else
-        {
-          glLineWidth ((GLfloat) 0.5); 
-          glPointSize ((GLfloat) 0.5); 
-        }
-      glDisable (GL_LINE_STIPPLE);
+      glLineWidth (S_thick); 
+      glPointSize (S_thick); 
     }
   else
     {
-      if (thick)
-        {
-          S_thick = thick;
-          if (style == 3)
-            /* dotted */
-            {
-              glLineStipple (thick, 0x5555);
-              glEnable (GL_LINE_STIPPLE);
-            }
-          else
-            /* dashed */
-            {
-              glLineStipple (thick, 0x1F1F);
-              glEnable (GL_LINE_STIPPLE);
-            }
-          glLineWidth ((GLfloat) thick); 
-          glPointSize ((GLfloat) thick); 
-        }
+      glLineWidth ((GLfloat) 0.5); 
+      glPointSize ((GLfloat) 0.5); 
+    }
+  if (style >= 5)
+    /* solid */
+    glDisable (GL_LINE_STIPPLE);
+  else
+    {
+      if (style == 3)
+        /* dotted */
+        glLineStipple (thick, 0x5555);
       else
-        {
-          glLineWidth ((GLfloat) 0.5); 
-          glPointSize ((GLfloat) 0.5); 
-        }
-     
+        /* dashed */
+        glLineStipple (thick, 0x1F1F);
+      glEnable (GL_LINE_STIPPLE);
     }
  GL_SetForeground (fg, FALSE);
 }
@@ -429,40 +408,28 @@ void GL_VideoInvert (int width, int height, int x, int y)
 void  GL_DrawEmptyRectangle (int fg, float x, float y, float width,
                              float height, float thick)
 { 
-  GL_SetForeground (fg, TRUE);
-  if (IS_ZERO (thick - 1))
-    {      
-      glBegin (GL_LINE_LOOP);
-      glVertex2f (x, y);
-      glVertex2f (x + width, y);
-      glVertex2f (x +  width, y + height);
-      glVertex2f (x, y + height);
-    }
-  else
-    {
-      thick = thick / 2;
-      glBegin (GL_QUADS);
-
-      glVertex2f (x - thick, y - thick);
-      glVertex2f (x + thick + width, y - thick);
-      glVertex2f (x + thick + width, y + thick);
-      glVertex2f (x - thick, y + thick);
-
-      glVertex2f (x - thick, y - thick + height);
-      glVertex2f (x + thick + width, y - thick + height);
-      glVertex2f (x + thick + width, y + thick + height);
-      glVertex2f (x - thick, y + thick + height);
-
-      glVertex2f (x - thick, y - thick);
-      glVertex2f (x + thick, y - thick);
-      glVertex2f (x + thick, y + thick + height);
-      glVertex2f (x - thick, y + thick + height);
-
-      glVertex2f (x - thick + width, y - thick);
-      glVertex2f (x + thick + width, y - thick);
-      glVertex2f (x + thick + width, y + thick + height);
-      glVertex2f (x - thick + width, y + thick + height);
-    }
+  thick = thick / 2.;
+  glBegin (GL_QUADS);
+  
+  glVertex2f (x - thick, y - thick);
+  glVertex2f (x + thick + width, y - thick);
+  glVertex2f (x + thick + width, y + thick);
+  glVertex2f (x - thick, y + thick);
+  
+  glVertex2f (x - thick, y - thick + height);
+  glVertex2f (x + thick + width, y - thick + height);
+  glVertex2f (x + thick + width, y + thick + height);
+  glVertex2f (x - thick, y + thick + height);
+  
+  glVertex2f (x - thick, y - thick);
+  glVertex2f (x + thick, y - thick);
+  glVertex2f (x + thick, y + thick + height);
+  glVertex2f (x - thick, y + thick + height);
+  
+  glVertex2f (x - thick + width, y - thick);
+  glVertex2f (x + thick + width, y - thick);
+  glVertex2f (x + thick + width, y + thick + height);
+  glVertex2f (x - thick + width, y + thick + height);
   glEnd ();
 }
 
@@ -481,37 +448,18 @@ void GL_DrawRectangle (int fg, float x, float y, float width, float height)
   glEnd ();
 }
 
-/*----------------------------------------------------------------------
-  GL_DrawLine
-  ----------------------------------------------------------------------*/
-void GL_DrawLine (int x1, int y1, int x2, int y2, ThotBool round)
-{
-  if (S_thick > 1 && round)
-    {
-      /* round line join*/
-      glBegin (GL_POINTS);
-      glVertex2i (x1, y1);
-      glVertex2i (x2, y2);
-      glEnd ();
-    }
-  glBegin (GL_LINES) ;
-  glVertex2i (x1, y1);
-  glVertex2i (x2, y2);
-  glEnd ();
-}
-
 /*----------------------------------------------------------------------
   GL_DrawSegments
   ----------------------------------------------------------------------*/
 void GL_DrawSegments (ThotSegment *point, int npoints)
 {
   int i;
-  
-  if (S_thick > 1)
+
+  if (S_thick > 1.)
     {
       glBegin (GL_POINTS); 
-      for(i=0; i < npoints; i++) 
-        {
+      for (i = 0; i < npoints; i++)
+       {
           glVertex2f ((point + i)->x1 , 
                       (point + i)->y1);
           glVertex2f ((point + i)->x2 , 
@@ -520,8 +468,8 @@ void GL_DrawSegments (ThotSegment *point, int npoints)
       glEnd ();
     }
   glBegin (GL_LINES) ;
-  for(i=0; i < npoints; i++) 
-    {
+  for (i = 0; i < npoints; i++)
+   {
       glVertex2f ((point + i)->x1 , 
                   (point + i)->y1);
       glVertex2f ((point + i)->x2 , 
@@ -536,37 +484,44 @@ void GL_DrawSegments (ThotSegment *point, int npoints)
 void GL_DrawArc (float x, float y, float w, float h, int startAngle,
                  int sweepAngle, ThotBool filled)
 {
-  GLint     i, slices;
-  PRECISION angleOffset;
-  PRECISION sinCache[SLICES_SIZE];
-  PRECISION cosCache[SLICES_SIZE];
-  PRECISION y_cache[SLICES_SIZE];
-  PRECISION x_cache[SLICES_SIZE];
-  PRECISION angle;
-  PRECISION fastx, fasty, width, height;
+  int       i, slices, npoints, j;
+  GLfloat   angleOffset;
+  GLfloat   sinCache[SLICES_SIZE];
+  GLfloat   cosCache[SLICES_SIZE];
+  GLfloat   cx, cy, angle, halfthick;
+  GLfloat   wr, hr, rx, ry;
+  ThotPoint points[SLICES_SIZE * 2 + 1];
 
-  width  = ((PRECISION)w) / 2;
-  height = ((PRECISION)h) / 2;
-  fastx  = ((PRECISION)x) + width; 
-  fasty  = ((PRECISION)y) + height;
+  halfthick = S_thick / 2.;
+  // set external ray
+  rx  = ((GLfloat)w) / 2.;
+  ry = ((GLfloat)h) / 2.;
+  // set ellipse center
+  cx  = ((GLfloat)x) + rx; 
+  cy  = ((GLfloat)y) + ry;
   if (w < 10 && h < 10)
-    {
-      glPointSize ((float)0.1);
       slices = 36;
-    }
   else
     slices = SLICES;
 
+  if (!filled && halfthick >= rx && halfthick >= ry)
+    {
+      // set external ray
+      filled = TRUE;
+      rx += halfthick;
+      ry += halfthick;
+    }
+
   startAngle = startAngle;
   sweepAngle = sweepAngle;
  
   /* Cache is the vertex locations cache */
-  angleOffset = (PRECISION) (startAngle / 180.0 * M_PI);
+  angleOffset = (GLfloat) (startAngle / 180.0 * M_PI);
   for (i = 0; i <= slices; i++) 
     {
-      angle = angleOffset + (PRECISION) ((M_PI * sweepAngle) / 180.0) * i / slices;
-      cosCache[i] = (PRECISION) DCOS(angle);
-      sinCache[i] = (PRECISION) DSIN(angle);
+      angle = angleOffset + (GLfloat) ((M_PI * sweepAngle) / 180.0) * i / ((GLfloat)slices);
+      cosCache[i] = (GLfloat) DCOS(angle);
+      sinCache[i] = (GLfloat) DSIN(angle);
     }
 
   if (fabs (sweepAngle - 360.0) < 0.0001) 
@@ -575,33 +530,49 @@ void GL_DrawArc (float x, float y, float w, float h, int startAngle,
       cosCache[slices] = cosCache[0];
     }
 
-  for (i = 0; i <= slices; i++)
-    {	
-      x_cache[i] = fastx + (width * cosCache[i]);
-      y_cache[i] = fasty - (height * sinCache[i]);
-    }
-
   if (filled)
     {
+      for (i = 0; i <= slices; i++)
+        {
+          points[i].x = cx + (rx * cosCache[i]);
+          points[i].y = cy - (ry * sinCache[i]);
+        }
+      points[i].x = points[0].x;
+      points[i].y = points[0].y;
+
       glBegin (GL_TRIANGLE_FAN);
       /* The center */
-      glVertex2d (fastx, fasty);
+      glVertex2d (cx, cy);
       for (i = 0; i <= slices; i++)
-        glVertex2d (x_cache[i], y_cache[i]);
+        glVertex2d (points[i].x, points[i].y);
       glEnd();
     }
-
-  if (!filled)
+  else
     {
-      if (w < 20 && h < 20)
-        glBegin(GL_POINTS);
-      else
-        glBegin(GL_LINE_STRIP);
-
-      slices--;
+      npoints = slices * 2 + 1;
+      if (halfthick < 1.)
+        halfthick = 1.;
+      // set internal ray
+      rx -= halfthick;
+      ry -= halfthick;
+      wr = rx + S_thick;
+      hr = ry + S_thick;
+      j = npoints;
       for (i = 0; i <= slices; i++)
-        glVertex2d (x_cache[i], y_cache[i]);
-      glEnd();
+        {
+          // external arc
+          points[i].x = cx + (rx * cosCache[i]);
+          points[i].y = cy - (ry * sinCache[i]);
+          // internal arc
+          points[j].x = cx + (wr * cosCache[i]);
+          points[j].y = cy - (hr * sinCache[i]);
+          j--;
+        }
+      npoints++;
+      points[npoints].x = points[slices+1].x;
+      points[npoints].y = points[slices+1].y;
+      // display a not convex polygon
+      MakefloatMesh (points, npoints);
     }
 }
 
@@ -609,27 +580,46 @@ void GL_DrawArc (float x, float y, float w, float h, int startAngle,
 
 /*----------------------------------------------------------------------
   GL_DrawLines
-  (Not static because used in geom.c)
+  npoints gives the number of points
   ----------------------------------------------------------------------*/
-void GL_DrawLines (ThotPoint *point, int npoints)
+void GL_DrawLines (ThotPoint *points, int npoints)
 {
   int i;
-  
-  if (S_thick > 1)
+
+ if (S_thick > 1.)
     {
       /* Joinning if a certain thickness ?*/
       glBegin (GL_POINTS); 
       for (i = 0; i<npoints; i++)
-        glVertex2dv ((double *) &point[i]);
+        glVertex2dv ((double *) &points[i]);
       glEnd (); 
     }
   glBegin (GL_LINE_STRIP);
   for (i = 0; i < npoints; i++)
-    glVertex2dv ((double *) &point[i]);
+    glVertex2dv ((double *) &points[i]);
   glEnd ();
   
 }
 
+/*----------------------------------------------------------------------
+  GL_DrawLine
+  ----------------------------------------------------------------------*/
+void GL_DrawLine (int x1, int y1, int x2, int y2, ThotBool round)
+{
+  if (S_thick > 1. && round)
+    {
+      /* round line join*/
+      glBegin (GL_POINTS);
+      glVertex2i (x1, y1);
+      glVertex2i (x2, y2);
+      glEnd ();
+    }
+  glBegin (GL_LINES) ;
+  glVertex2i (x1, y1);
+  glVertex2i (x2, y2);
+  glEnd ();
+}
+
 /*----------------------------------------------------------------------
   GL_DrawPolygon : tesselation handles 
   convex, concave and polygon with holes
@@ -882,8 +872,13 @@ static void DisplayViewBoxTransformation (PtrTransform Trans, int Width, int Hei
     {
       if (Trans->TransType == PtElViewBox)
         {
-          x_scale = (double) (Width / Trans->VbWidth);
-          y_scale = (double) (Height / Trans->VbHeight);
+          /* if it's an image with only a preserveAspectRatio attribute and no
+             viewBox attribute, Trans->VbWidth and Trans->VbHeight are negative
+             and the real size of the image should be used instead (TODO) */
+          if (Trans->VbWidth > 0)
+            x_scale = (double) (Width / Trans->VbWidth);
+          if (Trans->VbHeight > 0)
+            y_scale = (double) (Height / Trans->VbHeight);
           if (Trans->VbAspectRatio == ArNone)
             /* Do not force uniform scaling: the element's bounding box exactly
                matches the viewport rectangle */
@@ -970,7 +965,7 @@ static void DisplayViewBoxTransformation (PtrTransform Trans, int Width, int Hei
                   if (Width != s_width)
                     x_trans = (Width - s_width) / 2;
                   if (Height != s_height)
-                    y_trans = (Height - s_height);                  
+                    y_trans = (Height - s_height);
                 }
               else if (Trans->VbAspectRatio == ArXMaxYMax)
                 {
@@ -986,8 +981,8 @@ static void DisplayViewBoxTransformation (PtrTransform Trans, int Width, int Hei
           if (x_scale != 1 || y_scale != 1)
             is_scaled = TRUE;
 
-          x_trans += Trans->VbXTranslate;  
-          y_trans += Trans->VbYTranslate;
+          x_trans -= Trans->VbXTranslate * x_scale;  
+          y_trans -= Trans->VbYTranslate * y_scale;
           if (x_trans != 0 || y_trans != 0)
             is_translated = TRUE;
         }
