@@ -42,9 +42,10 @@ typedef struct _LockRelations
 static PtrLockRelations  DifferedChecks = NULL;
 static PtrLockRelations  ActiveChecks = NULL;
 static PtrAbstractBox    CheckedTable = NULL;
+static PtrAbstractBox    UpdatedParentTable = NULL;
 static ThotBool          Lock = FALSE;
-static ThotBool          DoUnlock1 = FALSE;
-static ThotBool          DoUnlock2 = FALSE;
+static ThotBool          DoUnlock1 = FALSE; // first unlock pass
+static ThotBool          DoUnlock2 = FALSE; // second unlock pass
 
 #include "appli_f.h"
 #include "attributes_f.h"
@@ -179,8 +180,14 @@ static ThotBool IsDifferredTable (PtrAbstractBox table, PtrAbstractBox cell,
             }
           else if (pLockRel->LockRTable[i] == table &&
                    (pLockRel->LockRCell[i] == NULL || pLockRel->LockRCell[i] == cell))
-            /* the table is already registered */
-            return TRUE;
+            {
+              /* the table is already registered */
+              if (DoUnlock2 && CheckedTable && ref == -1 &&
+                  UpdatedParentTable == NULL)
+                // the table is already checked but must be updated
+                UpdatedParentTable = table;
+              return TRUE;
+            }
           else if (DoUnlock1 && pLockRel->LockRTable[i] &&
                    !IsParentBox (table->AbBox, pLockRel->LockRTable[i]->AbBox))
             /* another differed table is enclosed by the same table */
@@ -972,6 +979,9 @@ static void CheckTableWidths (PtrAbstractBox table, int frame, ThotBool freely)
         extra = (pixels + n / 2) / n;
       else
         extra = pixels;
+      if (useMax)
+        // ignore extra pixels which will be added
+        delta -= pixels;
       for (cRef = 0; cRef < cNumber; cRef++)
         {
           box = colBox[cRef]->AbBox;
@@ -999,7 +1009,8 @@ static void CheckTableWidths (PtrAbstractBox table, int frame, ThotBool freely)
               else if (colPercent[cRef] < 0)
                 /* colPercent[cRef] = - new min */
                 i = var - colPercent[cRef];
-              else if (box->BxMinWidth + var > box->BxMaxWidth)
+              else if (colPercent[cRef] == 0 && colWidth[cRef] == 0 &&
+                       box->BxMinWidth + var > box->BxMaxWidth)
                 /* use the max instead of the min + delta */
                 i = box->BxMaxWidth;
               else
@@ -1011,13 +1022,14 @@ static void CheckTableWidths (PtrAbstractBox table, int frame, ThotBool freely)
           /* update the new inside width */
           if (addPixels && pixels > 0)
             {
-              if (pixels > extra && extra > 0)
+              if (pixels > extra && extra > 0 && cRef < cNumber - 1)
                 {
                   i += extra;
                   pixels -= extra;
                 }
               else
                 {
+                  // get all extra pixels
                   i += pixels;
                   pixels = 0;
                 }
@@ -1759,7 +1771,7 @@ static ThotBool SetCellWidths (PtrAbstractBox cell, PtrAbstractBox table,
             /* register a new differed table */
             DifferFormatting (table, cell, frame);
         }
-      else if (!IsDifferredTable (table, cell, frame))
+      else //if (!IsDifferredTable (table, cell, frame))
         /* something changed in the cell, check any table change */
         SetTableWidths (table, frame);
     }
@@ -1883,9 +1895,6 @@ void UpdateColumnWidth (PtrAbstractBox cell, PtrAbstractBox col, int frame)
       if (Lock)
         /* the table formatting is locked */
         DifferFormatting (table, cell, frame);
-      else if (IsDifferredTable (table, NULL, frame))
-        /* the table will be managed later */
-        return;
       else if (cell && cell->AbBox)
         {
           /* there a change within a specific cell */
@@ -1895,6 +1904,9 @@ void UpdateColumnWidth (PtrAbstractBox cell, PtrAbstractBox col, int frame)
               if (table->AbBox->BxCycles > 0)
                 printf ("table in progress\n");
 #endif
+              if (IsDifferredTable (table, NULL, frame))
+                /* the table will be managed later */
+                return;
               /* Now check the table size */
               CheckTableWidths (table, frame, TRUE);
               CheckRowHeights (table, frame);
@@ -2147,6 +2159,12 @@ void TtaUnlockTableFormatting ()
                   CheckTableWidths (table, pLockRel->LockRFrame[i], FALSE);
                   /* need to propagate to enclosing boxes */
                   ComputeEnclosing (pLockRel->LockRFrame[i]);
+                  while (UpdatedParentTable)
+                    {
+                      CheckedTable = UpdatedParentTable;
+                      UpdatedParentTable = NULL;
+                      CheckTableWidths (CheckedTable, pLockRel->LockRFrame[i], FALSE);
+                    }
                 }
               /* next entry */
               i++;
