@@ -30,6 +30,22 @@
 #include "edit_tv.h"
 #include "thotcolor_tv.h"
 
+#ifndef M_PI
+#define M_PI 3.14159265358979323846
+#endif
+#define M_PI_DOUBLE (6.2831853718027492)
+
+/* ((A)*(M_PI/180.0)) */
+#define DEG_TO_RAD(A)   ((float)A)/57.29577957795135
+#define RAD_TO_DEG(A)   ((float)A)*57.29577957795135
+
+/*If we should use a static table instead for
+  performance bottleneck...*/
+#define DCOS(A) ((float)cos (A))
+#define DSIN(A) ((float)sin (A))
+#define DACOS(A) ((float)acos (A))
+#define A_DEGREE 0.017453293
+
 #define ALLOC_POINTS    300
 
 #include "boxlocate_f.h"
@@ -727,70 +743,308 @@ static void ArrowDrawing (int frame, int x1, int y1, int x2, int y2,
 }
 
 /*----------------------------------------------------------------------
-  DrawArrow draw an arrow following the indicated direction in degrees :
-  0 (right arrow), 45, 90, 135, 180,
-  225, 270 ou 315.
-  parameter fg indicates the drawing color
+  DrawArrow draw an arrow.
+  orientation in degrees : 0 (right arrow), 45, 90, 135, 180, 225, 270 ou 315.
+  type : 0 = Arrow, 1 = Arrow with opposite directions, 2 = DoubleArrow,
+         3 = DoubleArrow with opposite directions
+         4 = two arrows with opposite directions, 5 = TeeArrow, 6 = ArrowBar
+         7 = Vector,  8 = Vector with opposite directions, 9 = TeeVector,
+         10 = VectorBar, 
+         11 = two vectors with opposite directions = Equilibrium
+         12 = ReverseVector, 13 = ReverseVector with opposite directions,
+         14 = TeeReverseVector, 15 = ReverseVectorBar
+         
+  fg : drawing color
   ----------------------------------------------------------------------*/
 void DrawArrow (int frame, int thick, int style, int x, int y, int l, int h,
-                int orientation, int fg)
+                int orientation, int type, int fg)
 {
-  int                 xm, ym, xf, yf;
+  int                 xm, ym, xf, yf, D1, D2;
+
+  if (fg < 0 || thick <= 0)return;
+  InitDrawing (style, thick, fg);
 
-  if (fg < 0)
-    return;
-  if (thick <= 0)
-    return;
   y += FrameTable[frame].FrTopMargin;
   xm = x + ((l - thick) / 2);
   xf = x + l - 1;
   ym = y + ((h - thick) / 2);
   yf = y + h - 1;
 
-  InitDrawing (style, thick, fg);
-  if (orientation == 0)
+  D1 = thick + 5;
+
+  /* Vector or ReverseVector */
+  if(type >= 12 && type <= 15)D2 = -D1; else D2 = D1;
+
+  /* Draw a Tee (type = 5, 9, 14) or a Bar (type = 6, 10, 15)  */
+  if(type == 5 || type == 6 || type == 9 || type == 10 || type == 14 || type == 15)
     {
-      /* draw a right arrow */
-      DoDrawOneLine (frame, x, ym, xf, ym);
-      ArrowDrawing (frame, x, ym, xf, ym, thick, fg);
+    switch(orientation)
+      {
+      case 0:
+        if(type == 5 || type == 9 || type == 14)DoDrawOneLine (frame, x, ym - D1, x, ym + D1);
+        else DoDrawOneLine (frame, xf, ym - D1, xf, ym + D1);
+      break;
+      case 90:
+        if(type == 5 || type == 9 || type == 14)DoDrawOneLine (frame, xm - D1, yf, xm + D1, yf);
+        else DoDrawOneLine (frame, xm - D1, y, xm + D1, y);
+      break;
+      case 180:
+        if(type == 5 || type == 9 || type == 14)DoDrawOneLine (frame, xf, ym - D1, xf, ym + D1);
+        else DoDrawOneLine (frame, x, ym - D1, x, ym + D1);
+      break;
+      case 270:
+        if(type == 5 || type == 9 || type == 14)DoDrawOneLine (frame, xm - D1, y, xm + D1, y);
+        else DoDrawOneLine (frame, xm - D1, yf, xm + D1, yf);
+      break;
+      default:
+      break;
+      }
     }
-  else if (orientation == 45)
+
+  switch(type)
     {
-      DoDrawOneLine (frame, x, yf, xf - thick + 1, y);
-      ArrowDrawing (frame, x, yf, xf - thick + 1, y, thick, fg);
-    }
-  else if (orientation == 90)
-    {
-      /* draw a bottom-up arrow */
-      DoDrawOneLine (frame, xm, y, xm, yf);
-      ArrowDrawing (frame, xm, yf, xm, y, thick, fg);
-    }
-  else if (orientation == 135)
-    {
-      DoDrawOneLine (frame, x, y, xf - thick + 1, yf);
-      ArrowDrawing (frame, xf - thick + 1, yf, x, y, thick, fg);
-    }
-  else if (orientation == 180)
-    {
-      /* draw a left arrow */
-      DoDrawOneLine (frame, x, ym, xf, ym);
-      ArrowDrawing (frame, xf, ym, x, ym, thick, fg);
-    }
-  else if (orientation == 225)
-    {
-      DoDrawOneLine (frame, x, yf, xf - thick + 1, y);
-      ArrowDrawing (frame, xf - thick + 1, y, x, yf, thick, fg);
-    }
-  else if (orientation == 270)
-    {
-      /* draw a top-down arrow */
-      DoDrawOneLine (frame, xm, y, xm, yf);
-      ArrowDrawing (frame, xm, y, xm, yf, thick, fg);
-    }
-  else if (orientation == 315)
-    {
-      DoDrawOneLine (frame, x, y, xf - thick + 1, yf);
-      ArrowDrawing (frame, x, y, xf - thick + 1, yf, thick, fg);
+    case 0: /* Arrow */
+    case 5: /* TeeArrow */
+    case 6: /* ArrowBar */
+      switch(orientation)
+        {
+        case 0:
+          /* draw a right arrow */
+          DoDrawOneLine (frame, x, ym, xf, ym);
+          ArrowDrawing (frame, x, ym, xf, ym, thick, fg);
+        break;
+        case 45:
+          DoDrawOneLine (frame, x, yf, xf - thick + 1, y);
+          ArrowDrawing (frame, x, yf, xf - thick + 1, y, thick, fg);
+        break;
+        case 90:
+          /* draw a bottom-up arrow */
+          DoDrawOneLine (frame, xm, y, xm, yf);
+          ArrowDrawing (frame, xm, yf, xm, y, thick, fg);
+        break;
+        case  135:
+          DoDrawOneLine (frame, x, y, xf - thick + 1, yf);
+          ArrowDrawing (frame, xf - thick + 1, yf, x, y, thick, fg);
+        break;
+        case 180:
+          /* draw a left arrow */
+          DoDrawOneLine (frame, x, ym, xf, ym);
+          ArrowDrawing (frame, xf, ym, x, ym, thick, fg);
+        break;
+        case 225:
+          DoDrawOneLine (frame, x, yf, xf - thick + 1, y);
+          ArrowDrawing (frame, xf - thick + 1, y, x, yf, thick, fg);
+        break;
+        case 270:
+          /* draw a top-down arrow */
+          DoDrawOneLine (frame, xm, y, xm, yf);
+          ArrowDrawing (frame, xm, y, xm, yf, thick, fg);
+        break;
+        case 315:
+          DoDrawOneLine (frame, x, y, xf - thick + 1, yf);
+          ArrowDrawing (frame, x, y, xf - thick + 1, yf, thick, fg);
+        break;
+        default:
+        break;
+        }
+    break;
+
+    case 1: /* Arrow with opposite directions */
+      switch(orientation)
+        {
+        case 0:
+          DoDrawOneLine (frame, x, ym, xf, ym);
+          ArrowDrawing (frame, x, ym, xf, ym, thick, fg);
+          ArrowDrawing (frame, xf, ym, x, ym, thick, fg);
+        break;
+
+        case 90:
+          DoDrawOneLine (frame, xm, y, xm, yf);
+          ArrowDrawing (frame, xm, yf, xm, y, thick, fg);
+          ArrowDrawing (frame, xm, y, xm, yf, thick, fg);
+        break;
+
+        default:
+        break;
+        }
+    break;
+
+    case 2: /* DoubleArrow */
+      switch(orientation)
+        {
+        case 0:
+          DoDrawOneLine (frame, x, ym - D1/2, xf - D1/2, ym - D1/2);
+          DoDrawOneLine (frame, x, ym + D1/2, xf - D1/2, ym + D1/2);
+          DoDrawOneLine (frame, xf - D1, ym - D1, xf, ym);
+          DoDrawOneLine (frame, xf - D1, ym + D1, xf, ym);
+        break;
+        case 90:
+          DoDrawOneLine (frame, xm + D1/2, y + D1/2, xm + D1/2, yf);
+          DoDrawOneLine (frame, xm - D1/2, y + D1/2, xm - D1/2, yf);
+          DoDrawOneLine (frame, xm, y, xm - D1, y + D1);
+          DoDrawOneLine (frame, xm, y, xm + D1, y + D1);
+        break;
+        case 180:
+          DoDrawOneLine (frame, x + D1/2, ym + D1/2, xf, ym + D1/2);
+          DoDrawOneLine (frame, x + D1/2, ym - D1/2, xf, ym - D1/2);
+          DoDrawOneLine (frame, x, ym, x + D1, ym - D1);
+          DoDrawOneLine (frame, x, ym, x + D1, ym + D1);
+        break;
+        case 270:
+          DoDrawOneLine (frame, xm + D1/2, y, xm + D1/2, yf - D1/2);
+          DoDrawOneLine (frame, xm - D1/2, y, xm - D1/2, yf - D1/2);
+          DoDrawOneLine (frame, xm, yf, xm - D1, yf - D1);
+          DoDrawOneLine (frame, xm, yf, xm + D1, yf - D1);
+        break;
+
+        default:
+        break;
+        }
+    break;
+
+    case 3: /* DoubleArrow with opposite directions */
+      switch(orientation)
+        {
+        case 0:
+          DoDrawOneLine (frame, x + D1/2, ym - D1/2, xf - D1/2, ym - D1/2);
+          DoDrawOneLine (frame, x + D1/2, ym + D1/2, xf - D1/2, ym + D1/2);
+          DoDrawOneLine (frame, xf - D1, ym - D1, xf, ym);
+          DoDrawOneLine (frame, xf - D1, ym + D1, xf, ym);
+          DoDrawOneLine (frame, x, ym, x + D1, ym - D1);
+          DoDrawOneLine (frame, x, ym, x + D1, ym + D1);
+        break;
+        case 90:
+          DoDrawOneLine (frame, xm + D1/2, y + D1/2, xm + D1/2, yf - D1/2);
+          DoDrawOneLine (frame, xm - D1/2, y + D1/2, xm - D1/2, yf - D1/2);
+          DoDrawOneLine (frame, xm, y, xm - D1, y + D1);
+          DoDrawOneLine (frame, xm, y, xm + D1, y + D1);
+          DoDrawOneLine (frame, xm, yf, xm - D1, yf - D1);
+          DoDrawOneLine (frame, xm, yf, xm + D1, yf - D1);
+        break;
+
+        default:
+        break;
+        }
+    break;
+
+    case 4: /* two arrows with opposite directions */
+      switch(orientation)
+        {
+        case 0:
+          DoDrawOneLine (frame, x, ym - D1/2, xf, ym - D1/2);
+          ArrowDrawing (frame, x, ym - D1/2, xf, ym - D1/2, thick, fg);
+          DoDrawOneLine (frame, x, ym + D1/2, xf, ym + D1/2);
+          ArrowDrawing (frame, xf, ym + D1/2, x, ym + D1/2, thick, fg);
+        break;
+        case 90:
+          DoDrawOneLine (frame, xm - D1/2, y, xm - D1/2, yf);
+          ArrowDrawing (frame, xm - D1/2, yf, xm - D1/2, y, thick, fg);
+          DoDrawOneLine (frame, xm + D1/2, y, xm + D1/2, yf);
+          ArrowDrawing (frame, xm + D1/2, y, xm + D1/2, yf, thick, fg);
+        break;
+        case 180:
+          DoDrawOneLine (frame, x, ym + D1/2, xf, ym + D1/2);
+          ArrowDrawing (frame, x, ym + D1/2, xf, ym + D1/2, thick, fg);
+          DoDrawOneLine (frame, x, ym - D1/2, xf, ym - D1/2);
+          ArrowDrawing (frame, xf, ym - D1/2, x, ym - D1/2, thick, fg);
+        break;
+        case 270:
+          DoDrawOneLine (frame, xm + D1/2, y, xm + D1/2, yf);
+          ArrowDrawing (frame, xm + D1/2, yf, xm + D1/2, y, thick, fg);
+          DoDrawOneLine (frame, xm - D1/2, y, xm - D1/2, yf);
+          ArrowDrawing (frame, xm - D1/2, y, xm - D1/2, yf, thick, fg);
+        break;
+        default:
+        break;
+        }
+    break;
+
+
+    case 7: /* Vector */
+    case 9: /* TeeVector */
+    case 10: /* VectorBar */
+    case 12: /* ReverseVector */
+    case 14: /* TeeReverseVector*/ 
+    case 15: /* ReverseVectorBar */
+      switch(orientation)
+        {
+        case 0:
+          DoDrawOneLine (frame, x, ym, xf, ym);
+          DoDrawOneLine (frame, xf - D1, ym - D2, xf, ym);
+        break;
+        case 90:
+          DoDrawOneLine (frame, xm, y, xm, yf);
+          DoDrawOneLine (frame, xm, y, xm - D2, y + D1);
+        break;
+        case 180:
+          DoDrawOneLine (frame, x, ym, xf, ym);
+          DoDrawOneLine (frame, x, ym, x + D1, ym - D2);
+        break;
+        case 270:
+          DoDrawOneLine (frame, xm, y, xm, yf);
+          DoDrawOneLine (frame, xm, yf, xm - D2, yf - D1);
+        break;
+        default:
+        break;
+        }
+    break;
+
+    case 8: /* Vector with opposite directions */
+    case 13: /* ReverseVector with opposite directions */
+      switch(orientation)
+        {
+        case 0:
+          DoDrawOneLine (frame, x, ym, xf, ym);
+          DoDrawOneLine (frame, x, ym, xf, ym);
+          DoDrawOneLine (frame, x + D1, ym - D2, x, ym);
+          DoDrawOneLine (frame, xf - D1, ym - D2, xf, ym);
+        break;
+        case 90:
+          DoDrawOneLine (frame, xm, y, xm, yf);
+          DoDrawOneLine (frame, xm, y, xm, yf);
+          DoDrawOneLine (frame, xm, y, xm - D2, y + D1);
+          DoDrawOneLine (frame, xm, yf, xm - D2, yf - D1);
+        break;
+
+        default:
+        break;
+        }
+    break;
+
+    case 11: /* two vectors with opposite directions */
+      switch(orientation)
+        {
+        case 0:
+          DoDrawOneLine (frame, x, ym - D1/2, xf, ym - D1/2);
+          DoDrawOneLine (frame, xf, ym - D1/2, xf - D1/2 - D1, ym - D1/2 - D1);
+          DoDrawOneLine (frame, x, ym + D1/2, xf, ym + D1/2);
+          DoDrawOneLine (frame, x, ym + D1/2, x + D1/2 + D1, ym + D1/2 + D1);
+        break;
+        case 90:
+          DoDrawOneLine (frame, xm - D1/2, y, xm - D1/2, yf);
+          DoDrawOneLine (frame, xm - D1/2, y, xm - D1/2 - D2, y + D1);
+          DoDrawOneLine (frame, xm + D1/2, y, xm + D1/2, yf);
+          DoDrawOneLine (frame, xm + D1/2, yf, xm + D1/2 + D1, yf - D1);
+        break;
+        case 180:
+          DoDrawOneLine (frame, x, ym + D1/2, xf, ym + D1/2);
+          DoDrawOneLine (frame, xf, ym + D1/2, xf - D1/2 - D1, ym + D1/2 + D1);
+          DoDrawOneLine (frame, x, ym - D1/2, xf, ym - D1/2);
+          DoDrawOneLine (frame, x, ym - D1/2, x + D1/2 + D1, ym - D1/2 - D1);
+        break;
+        case 270:
+          DoDrawOneLine (frame, xm + D1/2, y, xm + D1/2, yf);
+          DoDrawOneLine (frame, xm + D1/2, y, xm + D1/2 + D1, y + D1);
+          DoDrawOneLine (frame, xm - D1/2, y, xm - D1/2, yf);
+          DoDrawOneLine (frame, xm - D1/2, yf, xm - D1/2 - D1, yf - D1);
+        break;
+        default:
+        break;
+        }
+    break;
+
+    default:
+    break;
     }
 }
 
@@ -1954,9 +2208,11 @@ void DrawEllips (int frame, int thick, int style, int x, int y, int width,
   DrawHorizontalLine draw a horizontal line aligned top center or bottom
   depending on align value.
   The parameter fg indicates the drawing color.
+  leftslice and rightslice say if the left and right borders are sliced.
   ----------------------------------------------------------------------*/
 void DrawHorizontalLine (int frame, int thick, int style, int x, int y,
-                         int l, int h, int align, int fg, PtrBox box)
+                         int l, int h, int align, int fg, PtrBox box,
+                         int leftslice, int rightslice)
 {
   int        Y;
 
@@ -1989,22 +2245,60 @@ void DrawHorizontalLine (int frame, int thick, int style, int x, int y,
   DrawHat draw a hat aligned top
   The parameter fg indicates the drawing color.
   ----------------------------------------------------------------------*/
-void DrawHat (int frame, int thick, int style, int x, int y,
-              int l, int h, int align, int fg)
+void DrawHat (int frame, int thick, int style, int x, int y, int l, int h,
+              int fg, int direction)
 {
-  int        Y;
+int Y;
+
+h -= thick;
+l -= thick;
 
   if (thick > 0 && fg >= 0)
     {
-      y += FrameTable[frame].FrTopMargin;
-      y += (h - thick) / 2;
-      Y = y + (h - thick) / 2;
+      y += FrameTable[frame].FrTopMargin + h / 2;
+      Y = y + direction * h / 2;
       InitDrawing (style, thick, fg);
-      DoDrawOneLine (frame, x, Y, x + (l / 2), y);
-      DoDrawOneLine (frame, x + (l / 2), y, x + l - thick, Y);
+      DoDrawOneLine (frame, x, Y, x + l/2, y);
+      DoDrawOneLine (frame, x + l/2, y, x + l, Y);
     }
 }
 
+/*----------------------------------------------------------------------
+  DrawTilde draw a hat aligned top
+  The parameter fg indicates the drawing color.
+  ----------------------------------------------------------------------*/
+void DrawTilde (int frame, int thick, int style, int x, int y, int l, int h, int fg)
+{
+  int X, Y1, Y2, Xmax, Ymax;
+
+  h -= thick;
+  l -= thick;
+
+  if (thick > 0 && fg >= 0)
+    {
+      Xmax = 10;
+      Ymax = h / 3;
+      y += FrameTable[frame].FrTopMargin + h / 2;
+      InitDrawing (style, thick, fg);
+
+      for(X = 1, Y1 = 0; X <= Xmax; X++)
+        {     
+        Y2 = (int)(((float) Ymax) * DSIN (X*M_PI_DOUBLE/Xmax));
+        DoDrawOneLine (frame, x + (X-1)*l/Xmax, y + Y1, x + X*l/Xmax, y + Y2);
+        Y1 = Y2;
+        }
+    }
+}
+
+/*----------------------------------------------------------------------
+  DrawHorizontalBrace draw a horizontal brace aligned top or bottom
+  depending on align value.
+  The parameter fg indicates the drawing color.
+  ----------------------------------------------------------------------*/
+void DrawHorizontalParenthesis (int frame, int thick, int style, int x, int y,
+                          int l, int h, int align, int fg)
+{
+}
 
 /*----------------------------------------------------------------------
   DrawHorizontalBrace draw a horizontal brace aligned top or bottom
@@ -2039,13 +2333,46 @@ void DrawHorizontalBrace (int frame, int thick, int style, int x, int y,
     }
 }
 
+/*----------------------------------------------------------------------
+  DrawHorizontalBracket draw a horizontal brace aligned top or bottom
+  depending on align value.
+  The parameter fg indicates the drawing color.
+  ----------------------------------------------------------------------*/
+void DrawHorizontalBracket (int frame, int thick, int style, int x, int y,
+                          int l, int h, int align, int fg)
+{
+  int        Y;
+
+  if (thick > 0 && fg >= 0)
+    {
+      y += FrameTable[frame].FrTopMargin;
+      Y = y + (h - thick) / 2;
+      InitDrawing (style, thick, fg);
+      DoDrawOneLine (frame, x, Y, x + l, Y);
+      if (align == 0)
+        /* Over bracket */
+        {
+          DoDrawOneLine (frame, x, Y, x, y + h);
+          DoDrawOneLine (frame, x + l - thick, Y, x + l - thick, y + h);
+        }
+      else
+        /* Under bracket */
+        {
+          DoDrawOneLine (frame, x, Y, x, y);
+          DoDrawOneLine (frame, x + l - thick, Y, x + l - thick, y);
+        }
+    }
+}
+
 /*----------------------------------------------------------------------
   DrawVerticalLine draw a vertical line aligned left center or right
   depending on align value.
   parameter fg indicates the drawing color
+  topslice and bottomslice say if the top and bottom borders are sliced.
   ----------------------------------------------------------------------*/
 void DrawVerticalLine (int frame, int thick, int style, int x, int y,
-                       int l, int h, int align, int fg, PtrBox box)
+                       int l, int h, int align, int fg, PtrBox box,
+                       int topslice, int bottomslice)
 {
   int        X;
 
